
///////////////////////////////////////////////////////////////
//
//	File:		sequtil.c
//	Created:	20.12.2001
//	Purpose:	Routines which deal with sequences, matrices, banks 
//				read/write/convert operations
//	Comments:	
//	Authors:	Bogopolsky Gennady, EIMB (c) 2001
//
///////////////////////////////////////////////////////////////

#include "skoplib.h"
#include "windows.h"


//-------------------------------------------------------------
// Global data and type definition


// amino acid alphabet
const char*	amino_acids = "ARNDCQEGHILKMFPSTWYV";
char* amino_acids_3[] = { "ALA", "ARG", "ASN", "ASP", "CYS",
                          "GLN", "GLU", "GLY", "HIS", "ILE",
                          "LEU", "LYS", "MET", "PHE", "PRO",
                          "SER", "THR", "TRP", "TYR", "VAL"};

// map char to index in amino_acids array
// char_to_index['A'] = 0, ..., char_to_index['V'] = 12
char		char_to_index[255];
// amino acid substitution matrix
score_type	subst_matrix[ALPHABET_SIZE][ALPHABET_SIZE];
score_type	subst2_matrix[ALPHABET_SIZE][ALPHABET_SIZE];

score_type  seq_matrix[ALPHABET_SIZE * MAX_LEN];

//-------------------------------------------------------------
// forward declarations
int seq_make_from_alignstring(sequence* _seq, char* _alignstr);

//-------------------------------------------------------------
// Substitution matrix functionality

// reads substitution matrix into global subst_matrix array
// arguments: _filename - sz string with file name, "blosum62.txt" e.g.
// returns: 1 - ok, 0 - error
int matrix_read_from_file(char* _filename)
{
	FILE*	f;
	char*	szDelimiters = " \t";
	char*	ch;
	char	szBuff[256];
	int		i, l = 0, res = SUCCESS;

	f = fopen(_filename, "rt");
	if(!f) return E_FILEERROR;
	while(!feof(f) && l < 20)
	{
		fgets(szBuff, 255, f);
		ch = strtok(szBuff, szDelimiters);
		// if(!ch || *ch != "(!isalpha(ch[0]) && !isdigit(ch[0]))) continue;
		for(i = 0; i < 20 && ch;) {
			if(isdigit(ch[0]) || ch[0] == '-') {
				subst_matrix[l][i] = (score_type)atof(ch);
				i++;
			}
			ch = strtok(NULL, szDelimiters);
		};
		if(i < ALPHABET_SIZE) {
			res = E_INVALIDFORMAT; // error: the l-th string is shorter then it must be!!
			break;
		}
		if(i) l++;
		
	};
	fclose(f);
	return res;
}

// read of matrix in subst_matrix2
int matrix2_read_from_file(char* _filename)
{
	FILE*	f;
	char*	szDelimiters = " \t";
	char*	ch;
	char	szBuff[256];
	int		i, l = 0, res = SUCCESS;

	f = fopen(_filename, "rt");
	if(!f) return E_FILEERROR;
	while(!feof(f) && l < 20)
	{
		fgets(szBuff, 255, f);
		ch = strtok(szBuff, szDelimiters);
		// if(!ch || *ch != "(!isalpha(ch[0]) && !isdigit(ch[0]))) continue;
		for(i = 0; i < 20 && ch;) {
			if(isdigit(ch[0]) || ch[0] == '-') {
				subst2_matrix[l][i] = (score_type)atof(ch);
				i++;
			}
			ch = strtok(NULL, szDelimiters);
		};
		if(i < ALPHABET_SIZE) {
			res = E_INVALIDFORMAT; // error: the l-th string is shorter then it must be!!
			break;
		}
		if(i) l++;
		
	};
	fclose(f);
	return res;
}

//-------------------------------------------------------------
// Sequence & Bank

// initialize char_to_index array: A - 0, R - 1, N - 2, ..., V - 19
void init_char_to_index()
{
	unsigned	idx = 0;

	memset(char_to_index, 0, sizeof(char_to_index));
	while(idx < ALPHABET_SIZE) {
		char_to_index[amino_acids[idx]] = idx++;
	};
}
//--------------------------------------------------------------------
// READ FORM FSSP to malign_bank 
int malign_bank_read_from_FSSPfile(char* _filename, malign_bank* _bank, unsigned _cutoff_z){
	FILE*		f;
	char*		delims = " \t\r\n";
	int			l = 0, res = 1;
	char*		pos;
	char		buff[256], buff_z[8];
	unsigned	maxPos, posInd, maxSeq, seqInd, seqInd2, maxBlock, blockInd, initSeqIndex, endSeqIndex;
	double		z_score;
	char		work_char;

	f = fopen(_filename, "rt");
	if(!f) return E_FILEERROR;

	fgets(buff, 255, f); 
	while (!strstr(buff,"PDBID") && !feof(f)) fgets(buff, 255, f);
	pos = strtok(buff, delims);
	pos = strtok(NULL, delims);
	_bank->name = pos ? strdup(pos) : strdup(buff);

	//strcpy(_bank->name, pos);

	while (!strstr(buff,"SEQLENGTH") && !feof(f)) fgets(buff, 255, f);
	memset(buff,' ',strlen("SEQLENGTH")*sizeof(char));
	maxPos = atoi(buff);

	while (!strstr(buff,"NALIGN") && !feof(f)) fgets(buff, 255, f);
	memset(buff,' ',strlen("NALIGN")*sizeof(char));
	maxSeq = atoi(buff);
	maxBlock = (maxSeq-1)/30 + 1;
	//_bank->size = maxSeq;

	_bank->aligned_seq_strings = (char**)malloc(maxSeq * sizeof(char*));
	for (seqInd=0; seqInd<maxSeq; seqInd++) _bank->aligned_seq_strings[seqInd] = (char*)malloc((maxPos+1)*sizeof(char)); 
	_bank->seqs = (sequence*)malloc(maxSeq * sizeof(sequence));
	assert(_bank->aligned_seq_strings);
	assert(_bank->seqs);
	if(!_bank->aligned_seq_strings) return 0;// E_OUTOFMEMORY;
	if(!_bank->seqs) return 0;// E_OUTOFMEMORY;


	while (!strstr(buff,"SUMMARY") && !feof(f)) fgets(buff, 255, f);
	fgets(buff, 255, f);
	for (seqInd = 0; seqInd<maxSeq; seqInd++) {
		fgets(buff, 255, f);
		strncpy(buff_z, buff+19, 6);
		buff_z[6] = '\0';
		z_score = atof(buff_z);
		if (z_score < _cutoff_z) {
			for (seqInd2 = seqInd; seqInd2<maxSeq; seqInd2++) free(_bank->aligned_seq_strings[seqInd2]); 
			maxSeq = seqInd;
			_bank->aligned_seq_strings = (char**)realloc(_bank->aligned_seq_strings, maxSeq * sizeof(char*));
			_bank->seqs = (sequence*)realloc(_bank->seqs, maxSeq * sizeof(sequence));
			maxBlock = (maxSeq>0) ? (maxSeq-1)/30 + 1 : 0;
			break;
		}

		_bank->seqs[seqInd].name = (char*)malloc(6*sizeof(char));
		strncpy(_bank->seqs[seqInd].name,buff+13,5);
		_bank->seqs[seqInd].name[5] = '\0';
	}

	for (blockInd = 0; blockInd<maxBlock; blockInd++) {
		while (!strstr(buff,"ALIGNMENTS") && !feof(f)) fgets(buff, 255, f);
		fgets(buff, 255, f);
		initSeqIndex = blockInd*30;
		endSeqIndex = initSeqIndex+30;
		if (endSeqIndex > maxSeq) endSeqIndex = maxSeq;
		
		for (posInd=0; posInd<maxPos; posInd++) {
			fgets(buff, 255, f);
			for (seqInd = initSeqIndex; seqInd<endSeqIndex; seqInd++) {
				work_char = buff[42+3*(seqInd-initSeqIndex)];
				if ((work_char == 'a') || (work_char == 'b')) work_char = 'C';
				_bank->aligned_seq_strings[seqInd][posInd] = work_char;
			}
		}
	}
	for (seqInd=0; seqInd<maxSeq; seqInd++){ 
		_bank->aligned_seq_strings[seqInd][maxPos] = '\0'; 
		if((res = seq_make_from_alignstring(_bank->seqs + seqInd, _bank->aligned_seq_strings[seqInd])) != SUCCESS) return res;
	}

	fclose(f);
	_bank->size = maxSeq;
	return res;
}

//---------------------------------------------------------------------
// read from bank if you don't define earlier format
int read_bank_from_file(char* _filename, malign_bank* _bank, unsigned _z_cutoff){

	int res = 0;
	if (strstr(_filename, ".fssp")) res = malign_bank_read_from_FSSPfile(_filename, _bank, _z_cutoff);
// MR2006-2
	else 	if (strstr(_filename, ".TEST")) res=0; // res = malign_bank_read_from_FASTA(_bank);
	else res = malign_bank_read_from_file(_filename, _bank);
	
		res = malign_bank_read_from_file(_filename, _bank);
//		res = read_bank_as_fasta_from_file(_filename, _bank);
	return res;

}

//---------------------------------------------------------------------
// read from bank
int malign_bank_read_from_file(char* _filename, malign_bank* _bank)
{
	FILE*	f;
	char*	delims = " \t\r\n";
	char*	pos;
	char	buff[256];
	int		i, l = 0, res = 1;

	f = fopen(_filename, "rt");
	if(!f) return E_FILEERROR;

	// tittle
	fgets(buff, 255, f);
	pos = strtok(buff, "\r\n");
	pos = strchr(buff, ' ');
	//if(pos)
	_bank->name = pos ? strdup(pos + 1) : strdup(buff);

	//  empty strings
	do {
		fgets(buff, 255, f); 
	} while(strlen(buff) < 4);

	_bank->aligned_seq_strings = (char**)malloc(MAX_BANK * sizeof(char*));
	_bank->seqs = (sequence*)malloc(MAX_BANK * sizeof(sequence));
	assert(_bank->aligned_seq_strings);
	assert(_bank->seqs);
	if(!_bank->aligned_seq_strings) return E_OUTOFMEMORY1;
	if(!_bank->seqs) return E_OUTOFMEMORY1;
	
	// first go-go
	do {
		// name
		pos = strtok(buff, delims);
		_bank->seqs[l].name = strdup(pos);
		// amino acids
		pos = strtok(NULL, delims);
		_bank->aligned_seq_strings[l] = (char*)malloc(MAX_LEN * sizeof(char));
		memset(_bank->aligned_seq_strings[l], 0, MAX_LEN * sizeof(char));
		strcpy(_bank->aligned_seq_strings[l], pos);
		
		fgets(buff, 255, f);
		l++;
	} while(!feof(f) && strlen(buff) > 2);

	_bank->size = l;
	_bank->seqs = (sequence*)realloc(_bank->seqs, l * sizeof(sequence));
	_bank->aligned_seq_strings = (char**)realloc(_bank->aligned_seq_strings, l * sizeof(char*));

	// read a rest of sequences
	while(!feof(f)) {
		//  empty strings
		do { fgets(buff, 255, f); } while(!feof(f) && strlen(buff) < 4);
		
		for(i = 0; i < l; i++) {
			if(feof(f)) 
				return E_INVALIDFORMAT;
			// name
			pos = strtok(buff, delims);
			// amino acids
			pos = strtok(NULL, delims);
			strcat(_bank->aligned_seq_strings[i], pos);
			
			fgets(buff, 255, f);
		}
	}

	// realloc buffers and form sequence strings
	for(i = 0; i < l; i++) {
		_bank->aligned_seq_strings[i] =(char*) realloc(_bank->aligned_seq_strings[i], (strlen(_bank->aligned_seq_strings[i]) + 1) * sizeof(char));
		if((res = seq_make_from_alignstring(_bank->seqs + i, _bank->aligned_seq_strings[i])) != SUCCESS) return res;
	}

	fclose(f);
	return res;
}

//--------------------------------------------------------------------
int	merge_two_banks(malign_bank* _bank1, malign_bank* _bank2){

	int i;
	int bank_size = __min(_bank1->size,_bank2->size);
	

	for (i=0; i < bank_size; i++){
		_bank1->aligned_seq_strings[i] =(char*) realloc(_bank1->aligned_seq_strings[i], (strlen(_bank1->aligned_seq_strings[i]) + strlen(_bank2->aligned_seq_strings[i])+ 1) * sizeof(char));
		strcat(_bank1->aligned_seq_strings[i], _bank2->aligned_seq_strings[i]);

		_bank1->seqs[i].aas = (char*)realloc(_bank1->seqs[i].aas, (_bank1->seqs[i].length+_bank2->seqs[i].length+1) * sizeof(char));
		memcpy(_bank1->seqs[i].aas + (_bank1->seqs[i].length -1), _bank2->seqs[i].aas, (_bank2->seqs[i].length) * sizeof(char));
		//strcat(_bank1->seqs[i].aas, _bank2->seqs[i].aas);
		_bank1->seqs[i].length = _bank1->seqs[i].length + _bank2->seqs[i].length;
	}

	return SUCCESS;
}

//--------------------------------------------------------------------
int save_bank_as_clustal(malign_bank* _bank, char* _filename){

	FILE*		f;
	char		buff[51];
	unsigned	i,j, blok_numb, len;

	if (!_bank->size) return 0;

	if (strcmp("n", _filename)) strcpy(buff, _filename);
	   else {
		   strcpy(buff, _bank->name);
		   strcat(buff, "_n.aln");
	} 

	f = fopen(buff, "wt");
	if(!f) return E_FILEERROR;

	fprintf(f, "CLUSTAL\t%s\n", _bank->name);
	len = strlen(_bank->aligned_seq_strings[0]);
	blok_numb = (unsigned) len/50 +1;
	
	for (i=0; i<blok_numb; i++){
		fprintf(f,"\n");

		for (j=0; j<_bank->size; j++){
			strncpy(buff,_bank->aligned_seq_strings[j] + i*50, 50);
			buff[50] = '\0';
			fprintf(f, "%-12s %s\n", _bank->seqs[j].name, buff);
		}
	}

	fclose(f);

	return SUCCESS;
}

int  delete_seq_from_malign_bank(malign_bank* _bank, unsigned index){

	unsigned end_ind=_bank->size-1;

	if (index >= _bank->size) return -1 ;

	if (index < end_ind) {
		_bank->aligned_seq_strings[index]= (char*) realloc(_bank->aligned_seq_strings[index], sizeof(char) * (strlen(_bank->aligned_seq_strings[end_ind])+1));
		strcpy(_bank->aligned_seq_strings[index],_bank->aligned_seq_strings[end_ind]);

		_bank->seqs[index].name = (char*) realloc(_bank->seqs[index].name, sizeof(char) * (strlen(_bank->seqs[end_ind].name)+1));
		strcpy(_bank->seqs[index].name,_bank->seqs[end_ind].name);

		_bank->seqs[index].aas = (char*) realloc(_bank->seqs[index].aas, (_bank->seqs[end_ind].length +1) * sizeof(char));
		memcpy(_bank->seqs[index].aas,_bank->seqs[end_ind].aas, _bank->seqs[end_ind].length * sizeof(char));  
		_bank->seqs[index].length = _bank->seqs[end_ind].length;
	} 
	(_bank->size) --;
	free(_bank->aligned_seq_strings[end_ind]);
	free(_bank->seqs[end_ind].aas);
	free(_bank->seqs[end_ind].name);
	return SUCCESS;
}


//---------------------------------------------------------------------
// free all the memory that was allocated by seqs bank
void malign_bank_release(malign_bank* _bank)
{
	free(_bank->name);
	while(_bank->size--) {
		free(_bank->aligned_seq_strings[_bank->size]);
		free(_bank->seqs[_bank->size].aas);
		free(_bank->seqs[_bank->size].name);
	}
	free(_bank->seqs);
	free(_bank->aligned_seq_strings);
}

//---------------------------------------------------------------------
int fasta_bank_read_from_file(char* _filename, fasta_bank* _bank)
{
	FILE*	f;
	char*	delims = " \t\r\n";//"\t\r\n"; //
	char	buff[256], aa_buf[MAX_LEN];
	int		size, l = 0, res = 1, alloc_size = 0;

	f = fopen(_filename, "rt");
	if(!f) return E_FILEERROR;

	_bank->name = strdup(_filename);
	_bank->seqs = NULL;

	// read items
	fgets(buff, 255, f);
	do {
		if(buff[0] == '>') { // start of new sequence
			
			if(l >= alloc_size) {
				alloc_size += 16;
				_bank->seqs = (sequence*)realloc(_bank->seqs, alloc_size * sizeof(sequence));
				assert(_bank->seqs);
				if(!_bank->seqs) return E_OUTOFMEMORY1;
			}
			size = min (25,strcspn(buff, "\n")) - 1;
			_bank->seqs[l].name = (char*) malloc((size+1)*sizeof(char));
			if(IS_LINUX==1){
			strncpy(_bank->seqs[l].name, buff, size);
			 }
			else strncpy(_bank->seqs[l].name, buff+1, size);
			_bank->seqs[l].name = strdup(_bank->seqs[l].name);
                
		       //printf("%d  '%s'\n", size,_bank->seqs[l].name); 
		       

		//	_bank->seqs[l].name[size] = '\0';

		

                    
			//_bank->seqs[l].name = strdup(buff);
			memset(aa_buf, 0, sizeof(aa_buf));
			
			 //printf("%d  '%s'\n", size,_bank->seqs[l].name); 

			while(!feof(f)) {
				fgets(buff, 255, f);
				if (feof(f)) break;
				if(buff[0] != '>') {
					buff[strcspn(buff, delims)] = '\0';
					strcat(aa_buf, buff);
				} else break;
			}
			if( (res = seq_make_from_alignstring(_bank->seqs +l, aa_buf)) != SUCCESS) return res;
//			_bank->seqs[l].aas = strdup(aa_buf);
			_bank->seqs[l].length = strlen(aa_buf);
			l ++ ;
		} else {
			fgets(buff, 255, f);
		}
	} while(!feof(f));

	_bank->size = l;

	fclose(f);
	return res;
}
//----------------------------------------------------------------------
int read_fasta_resalign(char * file_name, char* align_str_1, char* align_str_2)
{
    char fast_del = '-';
	FILE*	f;
	char*	delims = " \t\r\n";
	char	buff[256];
	unsigned i;
	char	name1[32], name2[32], *name11, *name22;

	f = fopen(file_name, "rt");
	if(!f) return E_FILEERROR;

	fgets(buff, 255, f);
	while (!feof(f) && (buff[0] != '>' && buff[1] != '>')) 
		fgets(buff, 255, f);

	for(i = 0; i < 5; i++) fgets(buff, 255, f);

	strcpy(name1,strtok(buff,delims));
	strcpy(align_str_1,buff+7);
	align_str_1[strlen(align_str_1)-1]= '\0';

	fgets(buff, 255, f);fgets(buff, 255, f);
	strcpy(name2,strtok(buff,delims));
	strcpy(align_str_2,buff+7); 
	align_str_2[strlen(align_str_2)-1]= '\0';

	for(i = 0; i < 4; i++) fgets(buff, 255, f);

	while (!feof(f)){
		name11 = strtok(buff,delims);
		if (name11!= NULL && strcmp(name1, name11)==0){
			strcat(align_str_1,buff+7);
			align_str_1[strlen(align_str_1)-1]= '\0';
		} else break;

		fgets(buff, 255, f);fgets(buff, 255, f);
		name22 = strtok(buff,delims);
		if (name22!= NULL && strcmp(name2, name22)==0){
			strcat(align_str_2,buff+7);
			align_str_2[strlen(align_str_2)-1]= '\0';
		} else break;

		for(i = 0; i < 4; i++) fgets(buff, 255, f);
	}

	standardize_del_simbols(align_str_1, fast_del);
	standardize_del_simbols(align_str_2, fast_del);

	if (strlen(align_str_1) > strlen(align_str_2)) {
		for (i=strlen(align_str_2); i< strlen(align_str_1); i++) 
			align_str_2[i] =GAP_SYMBOL;
		align_str_2[i] = '\0';
	}
	if (strlen(align_str_1) < strlen(align_str_2)) {
		for (i=strlen(align_str_1); i< strlen(align_str_2); i++) 
			align_str_1[i] =GAP_SYMBOL;
		align_str_1[i] = '\0';
	}
	fclose(f);
	
	return SUCCESS;
}

//----------------------------------------------------
int read_blast_resalign(char * file_name, char* align_str_1, char* align_str_2){
 
	char fast_del = '-';
	FILE*	f;
	char*	delims = " \t\r\n";
	char	buff[256];
	//int		i;
	char	name1[32], name2[32], *name11, *name22;

	
	f = fopen(file_name, "rt");
	if(!f) return E_FILEERROR;

	fgets(buff, 255, f);
	while (!feof(f) && ((strstr(buff, "Query:")-buff)!=0)) fgets(buff, 255, f);

	if (feof(f)) { // no blast alignment 
		align_str_1 = '\0';
		align_str_2 = '\0';
		return -1;
	}

	strcpy(name1,strtok(buff,delims));
	strtok(NULL,delims);
	strcpy(align_str_1,strtok(NULL,delims));

	fgets(buff, 255, f);fgets(buff, 255, f);
	strcpy(name2,strtok(buff,delims));
	strtok(NULL,delims);
	strcpy(align_str_2,strtok(NULL,delims));

	fgets(buff, 255, f);fgets(buff, 255, f);
	while (!feof(f)){
		name11 = strtok(buff,delims);
		if (name11!= NULL && strcmp(name1, name11)==0){
			strtok(NULL,delims);
			strcat(align_str_1,strtok(NULL,delims));
		} else break;

		fgets(buff, 255, f);fgets(buff, 255, f);
		name22 = strtok(buff,delims);
		if (name22!= NULL && strcmp(name2, name22)==0){
			strtok(NULL,delims);
			strcat(align_str_2,strtok(NULL,delims));
		} else break;

		fgets(buff, 255, f);fgets(buff, 255, f);
	}
	standardize_del_simbols(align_str_1, fast_del);
	standardize_del_simbols(align_str_2, fast_del);

	fclose(f);
	
	return SUCCESS;
}


//----------------------------------------------------
void standardize_del_simbols(char* aln_str, char old_del){

	int i, len = strlen(aln_str);
	for (i=0; i<len; i++){
		if (aln_str[i] == old_del || aln_str[i] == ' ')
			aln_str[i] = GAP_SYMBOL;
	}
}

//-----------------------------------------------------------------------
int extend_algn_on_full_seqs(char* align_str_1, char* align_str_2,
							 sequence*	seq1  , sequence*	seq2,
							 int* path){
 
	sw_params	params;
	sequence	shot_seq1, shot_seq2;
	int			path1[MAX_LEN], path2rev[MAX_LEN],
				path_sh[MAX_LEN], fpath[MAX_LEN] ;
	int			res = 0;
	unsigned	len_sh, k;


	params.GEP = 0;
	params.GOP = 0;

	
	if (res = seq_make_from_alignstring(&shot_seq1,align_str_1)!= SUCCESS) return res;
		shot_seq1.name = (char*)malloc(5);
		strcpy(shot_seq1.name, "seq1");

	if (res = seq_make_from_alignstring(&shot_seq2,align_str_2)!= SUCCESS) return res;
		shot_seq2.name = (char*)malloc(5);
		strcpy(shot_seq2.name, "seq2");

	compute_sw_alignment(seq1,&shot_seq1, &params);		
	memcpy(path1, G_sw_compute_data.path, sizeof(G_sw_compute_data.path));
		//save_aln_asFASTA(path1, seq1,&shot_seq1, '.');

	compute_sw_alignment(&shot_seq2, seq2, &params);
	memcpy(path2rev, G_sw_compute_data.path, sizeof(G_sw_compute_data.path));
		//save_aln_asFASTA(path2rev, &shot_seq2, seq2, '.');
			
	format_alignment_path(
	  align_str_1,align_str_2,
	  path_sh, &len_sh);
	//save_aln_asFASTA(path_sh,&shot_seq1, &shot_seq2, '.');

	for(k=0; k < seq1->length; k++){
		if (path1[k]==-1) fpath[k] = -1;
		else {
			assert(path1[k] < len_sh);
			if (path_sh[path1[k]] == -1)fpath[k] = -1; 
			else {					
				assert(path_sh[path1[k]] < seq2->length);
				fpath[k]= path2rev[path_sh[path1[k]]];
			}
		}
	}
	memcpy(path, fpath,seq1->length * sizeof(int));

	free(shot_seq1.aas);free(shot_seq1.name); shot_seq1.length = 0;
	free(shot_seq2.aas);free(shot_seq2.name); shot_seq2.length = 0;

	return SUCCESS;
}

//-----------------------------------------------------------------------
int clustal_to_fasta(malign_bank* _bank_clustal, fasta_bank* _bank_fasta)
{
	unsigned i;

	_bank_fasta->name = _bank_clustal->name;
	_bank_clustal->name = NULL;
	_bank_fasta->seqs = (sequence*)malloc(_bank_clustal->size * sizeof(sequence));
	for(i = 0; i < _bank_clustal->size; i++) {
		_bank_fasta->seqs[i] = _bank_clustal->seqs[i];
		//memset(_bank_clustal->seqs + i, 0, sizeof(sequence));
		free(_bank_clustal->aligned_seq_strings[i]);
	}
	_bank_fasta->size = _bank_clustal->size;
	free(_bank_clustal->aligned_seq_strings);
	free(_bank_clustal->seqs);
	memset(_bank_clustal, 0, sizeof(malign_bank));

	return SUCCESS;
}

//----------------------------------------------------------
int read_bank_as_fasta_from_file(char* _filename, fasta_bank* _bank){

	int				res =0;
	malign_bank		bank_clustal;

	if(is_clustal_or_fssp(_filename)) { // clustal or FSSP format
		if((res = read_bank_from_file(_filename, &bank_clustal, 20)) != SUCCESS) {
		  printf("Error reading seqbank in clustal format %s \n", _filename);
		  return res;
		}
		clustal_to_fasta(&bank_clustal, _bank);

	} else { // fasta format

		if((res = fasta_bank_read_from_file(_filename, _bank)) != SUCCESS) {
		  printf("Error reading seqbank in fasta format %s \n", _filename);
		  return res;
		}
	}
	return SUCCESS;
}

//---------------------------------------------------------------------
void fasta_bank_release(fasta_bank* _bank)
{
	while(_bank->size -- ) {
		free(_bank->seqs[_bank->size].name);
		free(_bank->seqs[_bank->size].aas);
	}
	free(_bank->name);
	free(_bank->seqs);
	memset(_bank, 0, sizeof(fasta_bank));
}

//---------------------------------------------------------------------
int is_clustal(char* _file_name)
{
	FILE* f = fopen(_file_name, "rt");
	char buff[255];

	if(!f) return 0;

	fgets(buff, 255, f);
	fclose(f);

	if(strstr(buff, "CLUSTAL") != NULL) return 1;
	return 0;
}

//---------------------------------------------------------------------
int is_clustal_or_fssp(char* _file_name)
{
	FILE* f;
	char buff[255];

	if (strstr(_file_name, ".fssp")) return 1;

	f = fopen(_file_name, "rt");
	if(!f) return 0;

	fgets(buff, 255, f);
	fclose(f);

	if(strstr(buff, "CLUSTAL") != NULL) return 1;
	return 0;
}

//---------------------------------------------------------------
// read single seq from FASTA of PBD file

int read_seq_file (char* _file_name, sequence* _seq ){

	int		res = 0, pos;
	char	strand = ' ', *name, *tail;

	if (strstr(_file_name, "pdb") || strstr(_file_name, "ent")) {
		pos = strcspn(_file_name, " \t");
		name = strdup(strtok(_file_name, " \t"));
		strtok(NULL, " =\t");
		tail = strdup(strtok(NULL, " =\t"));
		if (tail != NULL) strand = tail[0];

		res = read_seq_PDB(name, _seq, strand);

		free(name);
		free(tail);
	} else {
		res = read_seq_fasta(_file_name, _seq);
	}
	return res;
}

//---------------------------------------------------------------------
// read single seq from FASTA format, 
// if it is a bank read only firtst seq
int read_seq_fasta (char* _file_name, sequence* _seq){

	FILE*	f;
	char*	delims = " \t\r\n";
	char	buff[256], aa_buf[MAX_LEN], pos;
	int		res = 0, size;//l = 0,  alloc_size = 0;

	f = fopen(_file_name, "rt");
	if(!f) return E_FILEERROR;

	fgets(buff, 255, f);
	if (buff[0] != '>') return E_FILEERROR;

	size = min (25,strcspn(buff, "\n")) - 1;
	_seq->name = (char*) malloc((size+1)*sizeof(char));
	strncpy(_seq->name, buff+1, size);
	_seq->name[size] = '\0';
	//_seq->name = strdup(buff + 1);
	
	memset(aa_buf, 0, sizeof(aa_buf));
	fgets(buff, 255, f);
	pos = strcspn(buff, delims);
	buff[pos] = '\0';

	if (feof(f) && (buff[0] != '>')) strcat(aa_buf, buff);
	while(!feof(f) && (buff[0] != '>')) {
		strcat(aa_buf, buff);
		fgets(buff, 255, f);
		buff[strcspn(buff, delims)] = '\0';
	}
	if( (res = seq_make_from_alignstring(_seq, aa_buf)) != SUCCESS) return res;
	//_seq->length = strlen(aa_buf);

	return SUCCESS;
}

//----------------------------------------------------------------------
// read single seq from PDB
int read_seq_PDB (char* _file_name, sequence* _seq, const char _strand){

	FILE*	f;
	char	strand = toupper(_strand);
	int		l = 0, res = 1, alloc_size = 0;
	char    str[MAX_LEN],str2[MAX_LEN], seq[MAX_LEN];
	char*	sh, s[32];
	char	am;
    unsigned   len, j;

	f = fopen(_file_name, "rt");
	if(!f)  {
	    printf("Error: File not found %s\n",_file_name);		
		return -100;
	}

	_seq->name = (char*)malloc(6*sizeof(char));

	do {
		fgets(str, sizeof(str), f);  
	} while (strstr(str,"DBREF") != str && !feof(f));
	
	if (feof(f))  {
	  fseek(f, 0, SEEK_SET);
	  if (sh = strrchr(_file_name, 92)) {
		  sh = sh+1;
		  sh = strtok(sh,".");
	  } else {
		  sh = strtok(_file_name,".");
	  }
	  if (strstr(sh,"pdb")) sh = sh+3;
	  strcpy(_seq->name, sh);
	  _seq->name[4] = strand;
	  _seq->name[5] = '\0';				
	} else {
	  strncpy(_seq->name, str+7,4);
	  _seq->name[4] = strand;
	  _seq->name[5] = '\0';				
	}

	do {
		fgets(str, sizeof(str), f);
	} while (strstr(str,"SEQRES") !=str && !feof(f));

	memset(seq,' ',sizeof(seq));
	len = 0;
	while ((strstr(str,"SEQRES")) && strand != str[11] && !feof(f)) {
	  fgets(str, sizeof(str), f);
	}
	if (feof(f) || !strstr(str,"SEQRES")) {

		if (strand == ' ') {
			strand = 'A';
			_seq->name[4] = strand;

			fseek(f, 0, SEEK_SET);
			do {
				fgets(str, sizeof(str), f);
			} while (strstr(str,"SEQRES") !=str && !feof(f));

			while (strstr(str,"SEQRES") && strand != str[11] && !feof(f)) {
			  fgets(str, sizeof(str), f);
			}
			 
			if (feof(f) || !strstr(str,"SEQRES")) {
				printf("Error: The protein has not strand '%c' nor ' ' in the file %s\n",strand,_file_name);
				return E_FILEERROR;
			}

		} else {
			printf("Error: The protein has not strand '%c' in the file %s\n",strand,_file_name);
			return E_FILEERROR;
		}
	}

	while (strstr(str,"SEQRES") && !feof(f) && strand == str[11]){

		strncpy(str2, str + 19, 51);
		str2[51] = '\0';

		for (j=0; j<13 ; j++) {
		   strncpy(s, str2 + 4*j, 3);
		   s[3] = '\0';
		   am = convert_3am_to_1index(s);
		   if (am == -1) break;

		   seq[len] = am;
		   len++;
		   assert(len < MAX_LEN);
		}
		fgets(str, sizeof(str), f);
	}

	seq[len] = '\0';
	_seq->aas = (char*)malloc( len * sizeof(char));
	memcpy(	_seq->aas, seq, len * sizeof(char));
	_seq->length = len;

	return SUCCESS;
}

//-------------------------------------------------------
// double of read_seq_PDB 
int read_seq_from_pdb (char* seq_name, 
					   char  strand,
					   sequence* seq_seq){


	FILE*	f;
	char	str[MAX_STR], str2[MAX_STR]; //, *sh;
	char	name[10], s[4];
	char    strand_new, am;
	char	seq[MAX_LEN] ;
	unsigned len, j;
	char	file_name[255];


	strcpy(file_name    , "pdb");
	strcpy(file_name + 3, seq_name);
	if (strlen(seq_name)!=4) printf("Length of PBD code is not equal 4 sibbols\nIt would be cuted!\n");

	strcpy(file_name + 7,".ent");

	f=fopen(file_name, "rt");
	strand = toupper(strand);

	if (f){
	  do fgets(str, sizeof(str), f);  
		 while (!strstr(str,"DBREF") && !feof(f));
  
	  if (feof(f))  {
		  fseek(f, 0, SEEK_SET);
		  strcpy(name, seq_name);
	  } else {
		   strncpy(name, str+7,4);
		   name[4] = '\0';
	  }
	  if (strand !=' ') {
		  name[4] = '_';
		  name[5] = strand;
		  name[6] = '\0';
	  }

	  do fgets(str, sizeof(str), f);
		while (strstr(str,"SEQRES")==NULL && !feof(f));

	  memset(seq,' ',sizeof(seq));
	  len = 0;
	  strand_new = str[11];

	  while (strstr(str,"SEQRES") && strand_new != strand && !feof(f)){
		strand_new = str[11];
		fgets(str, sizeof(str), f);
	  }
	  if (!strstr(str,"SEQRES") || feof(f)) {
		  fclose(f);
		  printf("Cannot find strand %c in file: %s\n", strand, file_name);
		  return 0;
	  }


  
	  while (strstr(str,"SEQRES") && !feof(f)){

		strand_new = str[11];
		if (strand_new != strand) {
		   /*fprintf( out_f, "> %s_%c length = %d\n",name, strand, len);
		   for (int k=0; k<len/50+1; k++) {
			  strncpy( seq_print, seq + k*50, 50);
			  seq_print[50] = '\0';
			  fprintf( out_f,"%s\n",seq_print);
		   }*/
			seq[len] = '\0';
			seq_make_from_alignstring(seq_seq, seq); 
			seq_seq->name = (char*)malloc((strlen(name)+1)*sizeof(char));
			strcpy(seq_seq->name, name);
		   break;
		}
		strncpy(str2, str + 19, 51);
		str2[51] = '\0';

		for (j=0; j<13 ; j++) {
		   strncpy(s, str2 + 4*j, 3);
		   s[3] = '\0';
		   am = convert_am_ac_to_1(s);
		   if (!am) break;

		   seq[len] = am;
		   len++;
		   assert(len < MAX_LEN);
		}
		fgets(str, sizeof(str), f);
	  }


	 fclose(f);
	} else {
	    printf("Cannot find file: %s\n", file_name);
		return 0;
	}
	return SUCCESS;
}
//-------------------------------------------------
// convert 3char alphabet to 1char
char convert_am_ac_to_1(char* s){

  int i;

	for (i=0; i < 20; i++){
     if (!strcmp(s, amino_acids_3[i])) {
        return amino_acids[i];
     }
  }

  if (isalpha(s[0]) && isalpha(s[1]) && isalpha(s[2])) return 'x';
  return 0;
}

//--------------------------------------------
char convert_3am_to_1(char* s){

   int i;
   for (i=0; i < 20; i++){
     if (!strcmp(s,amino_acids_3[i])) {
        return amino_acids[i];
     }
   }
   if (isalpha(s[0]) && isalpha(s[1]) && isalpha(s[2])) return 'x';
   return 0;
}

//--------------------------------------------
char convert_3am_to_1index(char* s){

   int i;
   for (i=0; i < 20; i++){
     if (!strcmp(s,amino_acids_3[i])) {
        return char_to_index[amino_acids[i]];
     }
   }
   if (isalpha(s[0]) && isalpha(s[1]) && isalpha(s[2])) return 0; // unidetified index
   return -1;
}

//------------------------------------------------------------
void seq_free(sequence* _seq){

	if (_seq->aas) { 
		free(_seq->aas);
		_seq->aas = NULL;
	}
	if (_seq->name) {
		free(_seq->name);
		_seq->name = NULL;
	}
	_seq->length = 0;
}
//------------------------------------------------------------
int  add_seq_in_fasta_bank(fasta_bank* _bank, sequence* _seq){

	int size = _bank->size;
	size++;

	_bank->seqs = (sequence*)realloc(_bank->seqs, size * sizeof(sequence));
	_bank->seqs[size-1].aas = (char*)malloc((_seq->length+1) *  sizeof(char));
	memcpy(_bank->seqs[size-1].aas, _seq->aas,_seq->length *  sizeof(char)); 

	_bank->seqs[size-1].length = _seq->length;
	_bank->seqs[size-1].name = strdup(_seq->name);
	seq_free(_seq);

	_bank->size = size;

	return SUCCESS;

}

//---------------------------------------------------------------------
// extracts sequence from aligned string _alignstr - aa with gap symbols
int seq_make_from_alignstring(sequence* _seq, char* _alignstr)
{
	unsigned idx = 0;
	char buff[MAX_LEN];

	while(*_alignstr != '\0') {
		if(isprint(*_alignstr) && *_alignstr != GAP_SYMBOL) buff[idx++] = char_to_index[toupper(*_alignstr)];
		_alignstr++;
	}
	_seq->aas = (char*)malloc(idx * sizeof(char));
	assert(_seq->aas);
	if(!_seq->aas) return E_OUTOFMEMORY1;
	memcpy(_seq->aas, buff, idx * sizeof(char));
	//_seq->aas[idx] = '\0';
	_seq->length = idx;

	return SUCCESS;
}

//---------------------------------------------------------------------
// path -> string alignment representation
int format_alignment_output(sequence* _seq1, sequence* _seq2, int* _path, char* _align1, char* _align2)
{
	unsigned i, j, m = _seq1->length, i_pos = 0, j_pos = 0;
    char * at1, * at2;
	at1= _align1; 
	at2= _align2; 
	assert(_seq1);
	assert(_seq2);
	assert(_align1);
	assert(_align2);

	for(i = 0; i < m; i++) {
		j = _path[i];
		// reach the (i, j)
		if(j == -1) {
			*_align1++ = amino_acids[_seq1->aas[i]];
			*_align2++ = GAP_SYMBOL;
		} else {
			while(j_pos < j)  {   // reach the j
				*_align1++ = GAP_SYMBOL;
				*_align2++ = amino_acids[_seq2->aas[j_pos++]];
			}
			*_align1++ = amino_acids[_seq1->aas[i]];
			*_align2++ = amino_acids[_seq2->aas[j_pos++]];
		}
	}
	while(j_pos < _seq2->length) {
		*_align1++ = GAP_SYMBOL;
		*_align2++ = amino_acids[_seq2->aas[j_pos++]];
	}
	*_align1 = '\0';
	*_align2 = '\0';
	return SUCCESS;
}

//---------------------------------------------------------------------
// Converts string representation of alignment to 
void format_alignment_path(char* _align1, char* _align2, int* _path, unsigned* _len)
{
	int pos1 = 0, pos2 = 0;

	while(*_align1 != '\0') {

		assert(*_align2 != '\0');

		if(isprint(*_align1) && isprint(*_align2)) {

			if(*_align1 != GAP_SYMBOL)
				if(*_align2 != GAP_SYMBOL) 
					_path[pos1++] = pos2++;
				else
					_path[pos1++] = -1;
			else 
				if(*_align2 != GAP_SYMBOL)
					pos2++;
		}

		_align1 ++ ;
		_align2 ++ ;
	}
	*_len = pos1;
}

//---------------------------------------------------------------------
// Calculates alignment identity _path_GS path of Golden standard alignment
// _path_sample - path of program alignment of he same sequences
float calc_alnide(int* _path_GS, int* _path_sample, unsigned _len_GS, unsigned* _match)
{
	unsigned i, count = 0, match = 0;

	for(i = 0; i < _len_GS; i++) {
		if(_path_GS[i] != -1) {
			if(_path_sample[i] == _path_GS[i]) match ++ ;
			count++;
		}	
	}
	if(_match) *_match = match;
	return count ? (float)match / count : 0.0f;
}

//---------------------------------------------------------------------
// Calculates alignment identity in kernels. _path_GS  - path of Golden standard alignment
// _path_sample - path of program alignment of he same sequences
float calc_kernel_alnide(int* _path_GS, int* _path_sample, unsigned _len_GS, unsigned* _mask, unsigned _length2)
{
	unsigned i, count = 0, match = 0;

	for(i = 0; i < _len_GS; i++) {
		if(_path_GS[i] != -1 && _mask[i * _length2 + _path_GS[i]]) {
			if(_path_sample[i] == _path_GS[i]) match ++ ;
			count++;
		}	
	}
	return count ? (float)match / count : 0.0f;
}

//---------------------------------------------------------------------
// Calculates alignment identity _path_GS path of Golden standard alignment
// _path_sample - path of program alignment of he same sequences
float calc_ide(char* _align1, char* _align2)
{
	int count = 0, match = 0;

	while(*_align1 != '\0' && *_align2 != '\0') {
		if(isprint(*_align1)) {
			assert(isprint(*_align2));
			if(*_align1 != GAP_SYMBOL && *_align2 != GAP_SYMBOL) {
				if(toupper(*_align1) == toupper(*_align2)) match++;
				count++;
			}
		}
		_align1 ++ ;
		_align2 ++ ;
	}

	return (float)match / count;
}

//---------------------------------------------------------------------
// Calculates number of matches in alignment
unsigned calc_match_count(int* _path, unsigned _length)
{
	unsigned count = 0, i;
	for(i = 0; i < _length; i++) 
		if(_path[i] != -1) count++;
	return count;
}

//--------------------------------------------------------------------------------
unsigned * create_mask(sequence* _seq1, sequence* _seq2, kernel* _kernels, unsigned _kernel_count)
{
	unsigned  m = _seq1->length, n = _seq2->length, i, j, k;
	coord_type l;
	unsigned* mask = (unsigned*)malloc(m * n * sizeof(unsigned));
	assert(mask);
	if(!mask) return NULL;
	memset(mask, 0, m * n * sizeof(unsigned));

	// setup mask
	for(k = 0; k < _kernel_count; k++) {
		i = _kernels[k].x;
		j = _kernels[k].y;
		l = 0;
		while(l < _kernels[k].length) {
			mask[i * n + j] = k + 1;
			i++; j++; l++;
		}
	}
	return mask;
}

//--------------------------------------------------------------------------------
void process_alignment(sequence* _seq1, sequence* _seq2, int* _path, int* _path_GS, 
					   unsigned* _mask, unsigned* _used_count, unsigned* _GS_count, unsigned* _corr_count)
{
	int m = _seq1->length, n = _seq2->length, k, l;
	int skop_kernels[MAX_KERNELS], skop_kernel_count = 0, gs_kernels[MAX_KERNELS], gs_kernel_count = 0;
	memset(skop_kernels, 0, sizeof(skop_kernels));
	memset(gs_kernels, 0, sizeof(gs_kernels));

	// go among the skop alignment path
	for(l = 0; l < m; l++) {
		if(_path[l] != -1) {
			k = l * n + _path[l];
			if(_mask[k] > 0 && _mask[k] != skop_kernels[skop_kernel_count?skop_kernel_count-1:0]) {
				skop_kernels[skop_kernel_count ++ ] = _mask[k];
				assert(skop_kernel_count < MAX_KERNELS);
			}
		}
		if(_path_GS[l] != -1) {
			k = l * n + _path_GS[l];
			if(_mask[k] > 0 && _mask[k] != gs_kernels[gs_kernel_count?gs_kernel_count-1:0]) {
				gs_kernels[gs_kernel_count ++ ] = _mask[k];
				assert(gs_kernel_count < MAX_KERNELS);
			}
		}
	}

	// now comparison
	*_corr_count = 0;
	for(l = 0; l < skop_kernel_count; l++) {
		for(k = 0; k < gs_kernel_count; k++) {
			if(skop_kernels[l] == gs_kernels[k]) {
				(*_corr_count) ++;
				break;
			}
		}
	}
	if(_GS_count) *_GS_count = gs_kernel_count;
	if(_used_count) *_used_count = skop_kernel_count;
}



 //------------------------------------------------------------------------------------
 int print_alignment_ext(sequence* _seq1, sequence* _seq2, int* _path_GS, int* _path_skop, kernel* _kernels, unsigned _count, unsigned* _mask, FILE* _f_out)
 {
  const int* representation_1 = _path_skop;
  const int* representation_2 = _path_GS;

  int NumberKerns;
  char c1,c2;
  char aln_seq_1[MAX_STR], out_seq_1[MAX_STR];
  char aln_seq_2[MAX_STR], out_seq_2[MAX_STR];
  char compare_str[MAX_STR], *compare_pos, *aln1_pos, *aln2_pos;
  int posIndex = 0, wz;
  int max_length, seqIndex, i = 0, j = 0, k = 0;

  char Markers[MAX_KERNEL_IN_ALI] ="?ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
  int KernIndexs[MAX_KERNEL_IN_ALI]= {-1,-1};
  int currentKernIndex = 0;
  char str_kerns[MAX_STR], *pos_kerns;
  char szDpi1[1024];
  char szDpi2[1024];
  char szUsedGS[1024],szUsed[1024] ;

  score_type used_score;

  int len_1 = _seq1->length;
  int len_2 = _seq2->length;

  int pos_index_1 = -1;
  int pos_index_2 = -1;

  char res_1;
  char res_2;

  memset(compare_str, 0, sizeof(compare_str));
  memset(aln_seq_1, 0, sizeof(aln_seq_1));
  memset(aln_seq_2, 0, sizeof(aln_seq_2));

  format_alignment_output(_seq1, _seq2, _path_skop, aln_seq_1, aln_seq_2);

  max_length = strlen(aln_seq_1);
  compare_pos = compare_str;

  for (seqIndex = 0; seqIndex<max_length; seqIndex++) {
    if (aln_seq_1[seqIndex] != '.' ) {
        if ((representation_1[posIndex] == representation_2[posIndex]) && (representation_1[posIndex] != -1)) {
           *compare_pos++ = '_';
        }
         else *compare_pos++ = ' ';
        posIndex ++;
    }
     else *compare_pos++ = ' ';
  }

  fprintf(_f_out,"%s\n", compare_str);

  aln1_pos = out_seq_1;
  aln2_pos = out_seq_2;

  for (k=0; k<=max_length; k++){

    c1 = toupper(aln_seq_1[k]);
    c2 = toupper(aln_seq_2[k]);

    if (subst_matrix[char_to_index[c1]][char_to_index[c2]] <= 0) {
      c1 = tolower(c1);
      c2 = tolower(c2);
    }
    *aln1_pos++ = c1;
    *aln2_pos++ = c2;
  }

  fprintf(_f_out, "%s\n%s\n", out_seq_1, out_seq_2);
/*
  pos_kerns = str_kerns;
  memset(str_kerns, 0, sizeof(str_kerns));
  for (posIndex = 0; posIndex<max_length; posIndex++) {

   res_1 = aln_seq_1[posIndex];
   res_2 = aln_seq_2[posIndex];

   if (res_1 != GAP_SYMBOL) pos_index_1++;
   if (res_2 != GAP_SYMBOL) pos_index_2++;

   if ((pos_index_1>=0 && pos_index_2>=0) && (pos_index_1<len_1 && pos_index_2<len_2)) {
    if((_mask[pos_index_1 * len_2 + pos_index_2]) &&
      ((res_1 != GAP_SYMBOL) && (res_2 != GAP_SYMBOL))) {

	  currentKernIndex = _mask[pos_index_1 * len_2 + pos_index_2];
      if (currentKernIndex == KernIndexs[i]) {
		  if (compare_str[posIndex]==' ') { *pos_kerns++ = tolower(Markers[i]); }
          else *pos_kerns++ = Markers[i];
      }
      else {
           i++;
           if (i>MAX_KERNEL_IN_ALI) {
             printf("ERROR !!! Too long alignment %d\n ", i );
             exit(1);
           }
         KernIndexs[i] = currentKernIndex;
         if (compare_str[posIndex]==' ') { *pos_kerns++ = tolower(Markers[i]);}
          else *pos_kerns++ = Markers[i];
      }
    }
    else {
     *pos_kerns++ = GAP_SYMBOL;
    }
   }
   else {
    *pos_kerns++ = GAP_SYMBOL;
   }
  }
  NumberKerns = i;

  fprintf(_f_out, "%s\n\n", str_kerns);
*/
/*
  // Print Kernels
  for (i = 1; i <= NumberKerns; i++) {
    wz = KernIndexs[i] - 1;
   	if(wz < 0) continue;
    //fprintf(_f_out, "%3d (%c) (%d,%d) len = %d score= %f\t Used_score= ", i, Markers[i], _kernels[wz].x, _kernels[wz].y, _kernels[wz].length, _kernels[wz].score);
	used_score = 0;

	//   _kernels[wz - 1]
	for (j = 0; j < _kernels[wz].length; j++) {
		c1 = amino_acids[_seq1->aas[_kernels[wz].x + j]];
		c2 = amino_acids[_seq2->aas[_kernels[wz].y + j]];
		
		if (_path_skop[(_kernels[wz].x + j)] == (_kernels[wz].y + j)){
				szUsed[j] = '-';
				used_score += subst_matrix[char_to_index[c1]][char_to_index[c2]];
		} else szUsed[j] = ' ';
		
		if (subst_matrix[char_to_index[c1]][char_to_index[c2]]<=0){
		//if (c1 != c2){
			c1 = tolower(c1);
			c2 = tolower(c2);
		}
		szDpi1[j] = c1;
		szDpi2[j] = c2;
		szUsedGS[j] = _path_GS[(_kernels[wz].x + j)] == (_kernels[wz].y + j) ? '_' : ' ';
	}
	szDpi1[j] = '\0';
	szDpi2[j] = '\0';
	szUsed[j] = '\0'; szUsedGS[j] = '\0';

    fprintf(_f_out, "%3d (%c) (%d,%d) len = %d score= %.2f\t Used_score= %.2f\n", i, Markers[i], _kernels[wz].x, _kernels[wz].y, _kernels[wz].length, _kernels[wz].score, used_score);
	fprintf(_f_out, "%s\n%s\n%s\n%s\n\n", szUsedGS, szDpi1, szDpi2, szUsed);

  }
*/
  fprintf(_f_out, "==========================================\n");

  return 0;
}

//-----------------------------------------------------------------
int island_compare_with_kernels(sequence* _seq1, sequence* _seq2, 
								int* _path, int* _pathGS, 
								unsigned* _mask,
								kernel* _kernels, unsigned _kernel_count,
								islands_stat* isl_stat){

	int pos = 0, isl_nom = -1, isl_len, isl_sim ;
	char	 is_isl = 0, is_kern = 0, start_max;
	int		 kern_ind = -1, new_kern_ind = -1, max_kern_ind=-1, kern_count =0 ; 
	unsigned len1 = _seq1->length, len2 = _seq2->length;
	score_type	isl_score, max_score, curr_max_score, add_score;
	score_type  kern_sum_weight, kern_weight, kern_max_weight ;

	while (pos<len1){

		if (!is_isl){			
			if (_pathGS[pos] != -1) { // begin of island
				is_isl  = 1;
				isl_nom ++;
				
				is_kern = 0; 
				isl_len = 1; isl_sim = 0;
				isl_score = max_score= 0;
				start_max = 0;
				kern_ind = max_kern_ind = -1; kern_count =0 ;
				kern_sum_weight= 0;
				kern_max_weight= -1000;
				kern_weight = -1000;

				add_score = subst_matrix[_seq1->aas[pos]][_seq2->aas[_pathGS[pos]]];
				isl_score += add_score;

				if (_path[pos] == _pathGS[pos]) isl_sim++;
				if (_mask[pos * len2 + _pathGS[pos]]) {
					is_kern = 1;
					kern_ind =_mask[pos * len2 + _pathGS[pos]] -1;// is_in_kernel(pos,_pathGS[pos],_kernels,_kernel_count);					  
					assert (kern_ind != -1);
					kern_count = 1;
					kern_sum_weight = add_score;
					kern_weight = add_score;
					max_kern_ind = kern_ind;
				}

				if (add_score >0) {
					max_score = curr_max_score = add_score; 
					start_max = 1;
				}
			}
		} else {
			// end of island
			if ( (_pathGS[pos] == -1) || (_pathGS[pos-1]!=-1 && _pathGS[pos-1]+1 != _pathGS[pos])){ 
				is_isl=0;

				if (curr_max_score > max_score) max_score = curr_max_score;

				// save statistik for ended island
				isl_stat[isl_nom].isl_index= isl_nom;  
				isl_stat[isl_nom].isl_len = isl_len;
				isl_stat[isl_nom].isl_sim = isl_len ? (double) isl_sim/isl_len : 0;
				isl_stat[isl_nom].have_kern = is_kern;
				isl_stat[isl_nom].isl_score =isl_score;
				isl_stat[isl_nom].max_score =max_score;


				if (kern_weight > kern_max_weight){
					kern_max_weight = kern_weight;
					max_kern_ind = kern_ind;
				}

				isl_stat[isl_nom].kernel_count = kern_count;
				isl_stat[isl_nom].kern_max_weight = kern_max_weight;
				isl_stat[isl_nom].kern_full_weight = (max_kern_ind!=-1) ? _kernels[max_kern_ind].score : -1;
				isl_stat[isl_nom].kern_sum_weight = kern_sum_weight;

				if (_pathGS[pos]!=-1) pos--; //begin new island					

			} else { // continue of island
				isl_len++;
				add_score = subst_matrix[_seq1->aas[pos]][_seq2->aas[_pathGS[pos]]];
				isl_score += add_score;

				if (_path[pos] == _pathGS[pos]) isl_sim++;
			
				if (_mask[pos * len2 + _pathGS[pos]]) {
					is_kern = 1;
					new_kern_ind = _mask[pos * len2 + _pathGS[pos]] -1;//is_in_kernel(pos,_pathGS[pos],_kernels,_kernel_count);					  
					assert (new_kern_ind != -1);
					kern_sum_weight += add_score;
					
					if (new_kern_ind == kern_ind){ // the same kernel
						kern_weight += add_score;

					} else { // new kernel
						kern_count++;
						if (kern_weight > kern_max_weight){
							kern_max_weight = kern_weight;
							max_kern_ind = kern_ind;
						}

						kern_ind = new_kern_ind;
						kern_weight = add_score;
					}
				}
												
				if (start_max) {
					curr_max_score += add_score;
					if (curr_max_score > max_score) max_score = curr_max_score;
					else {
						if (curr_max_score <= 0) {
							curr_max_score = 0;
							start_max = 0;
						}
					}
				} else {
					if (add_score >0) {
						curr_max_score = add_score; 
						start_max = 1;
					}
				}
			}
		}
		pos++;
	}
	if ( is_isl) { // save last unended island
		if (curr_max_score > max_score) max_score = curr_max_score;

		isl_stat[isl_nom].isl_index= isl_nom;  
		isl_stat[isl_nom].isl_len = isl_len;
		isl_stat[isl_nom].isl_sim = isl_len ? (double) isl_sim/isl_len : 0;
		isl_stat[isl_nom].have_kern = is_kern;
		isl_stat[isl_nom].isl_score =isl_score;
		isl_stat[isl_nom].max_score =max_score;

		if (kern_weight > kern_max_weight){
					kern_max_weight = kern_weight;
					max_kern_ind = kern_ind;
				}
				isl_stat[isl_nom].kernel_count = kern_count;
				isl_stat[isl_nom].kern_max_weight = kern_max_weight;
				isl_stat[isl_nom].kern_full_weight = (max_kern_ind!=-1) ? _kernels[max_kern_ind].score : -1;
				isl_stat[isl_nom].kern_sum_weight = kern_sum_weight;
	}

	return isl_nom +1;
}

int print_alignment_ext_MR(sequence* _seq1, sequence* _seq2, int* _path_skop,FILE* _f_out)
 {
  const int* representation_1 = _path_skop;
//  const int* representation_2 = _path_GS;

  int NumberKerns;
  char c1,c2;
  char aln_seq_1[MAX_STR], out_seq_1[MAX_STR], out_buf_1[MAX_STR];
  char aln_seq_2[MAX_STR], out_seq_2[MAX_STR], out_buf_2[MAX_STR];

  char compare_str[MAX_STR], *compare_pos, *aln1_pos, *aln2_pos;
  int posIndex = 0, wz;
  int max_length, seqIndex, i = 0, j = 0, k = 0;
  int StrLenAli=60;
  int iz, iz1, izr, izp;

  char Markers[MAX_KERNEL_IN_ALI] ="?ABCDEFGHIJKLMNOPQRSTUVWXYZ123456789+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
  int KernIndexs[MAX_KERNEL_IN_ALI]= {-1,-1};
  int currentKernIndex = 0;
  char str_kerns[MAX_STR], *pos_kerns;
  char szDpi1[1024];
  char szDpi2[1024];
  char szUsedGS[1024],szUsed[1024] ;

  score_type used_score;

  int len_1 = _seq1->length;
  int len_2 = _seq2->length;

  int pos_index_1 = -1;
  int pos_index_2 = -1;

  char res_1;
  char res_2;

  memset(compare_str, 0, sizeof(compare_str));
  memset(aln_seq_1, 0, sizeof(aln_seq_1));
  memset(aln_seq_2, 0, sizeof(aln_seq_2));

  format_alignment_output(_seq1, _seq2, _path_skop, aln_seq_1, aln_seq_2);

  max_length = strlen(aln_seq_1);
  compare_pos = compare_str;

  // Stroka sravneniya s GS
/*
  for (seqIndex = 0; seqIndex<max_length; seqIndex++) {
    if (aln_seq_1[seqIndex] != '.' ) {
        if ((representation_1[posIndex] == representation_2[posIndex]) && (representation_1[posIndex] != -1)) {
           *compare_pos++ = '_';
        }
         else *compare_pos++ = ' ';
        posIndex ++;
    }
     else *compare_pos++ = ' ';
  }

  fprintf(_f_out,"%s\n", compare_str);
*/

  aln1_pos = out_seq_1;
  aln2_pos = out_seq_2;

  for (k=0; k<=max_length; k++){

    c1 = toupper(aln_seq_1[k]);
    c2 = toupper(aln_seq_2[k]);

    if (subst_matrix[char_to_index[c1]][char_to_index[c2]] <= 0) {
      c1 = tolower(c1);
      c2 = tolower(c2);
    }
    *aln1_pos++ = c1;
    *aln2_pos++ = c2;
  }

  fprintf(_f_out, "\n");

  iz1=0;
  for (iz=0; iz < max_length; iz++){
	  out_buf_1[iz1]= out_seq_1[iz];
	  out_buf_2[iz1]= out_seq_2[iz];
	  iz1++;
	  izr = iz%StrLenAli;
	  if( ( izr ==0) && (iz > 0) )  { 
//	  if ( (iz%StrLenAli)==0) && (iz > 0) ){
		for (izp=0; izp < iz1; izp++){  
          fprintf(_f_out, "%c", out_buf_1[izp]);
		}
		fprintf(_f_out, "\n");
		for (izp=0; izp < iz1; izp++){  
          fprintf(_f_out, "%c", out_buf_2[izp]);
		}
		fprintf(_f_out, "\n");
		fprintf(_f_out, "\n");
        iz1=0;
	  }
  }
  for (izp=0; izp < iz1; izp++){  
    fprintf(_f_out, "%c", out_buf_1[izp]);
  }
    fprintf(_f_out, "\n");
  for (izp=0; izp < iz1; izp++){  
    fprintf(_f_out, "%c", out_buf_2[izp]);
  }
  fprintf(_f_out, "\n");
  fprintf(_f_out, "\n");

  /*
  pos_kerns = str_kerns;
  memset(str_kerns, 0, sizeof(str_kerns));
  for (posIndex = 0; posIndex<max_length; posIndex++) {

   res_1 = aln_seq_1[posIndex];
   res_2 = aln_seq_2[posIndex];

   if (res_1 != GAP_SYMBOL) pos_index_1++;
   if (res_2 != GAP_SYMBOL) pos_index_2++;

   if ((pos_index_1>=0 && pos_index_2>=0) && (pos_index_1<len_1 && pos_index_2<len_2)) {
    if((_mask[pos_index_1 * len_2 + pos_index_2]) &&
      ((res_1 != GAP_SYMBOL) && (res_2 != GAP_SYMBOL))) {

	  currentKernIndex = _mask[pos_index_1 * len_2 + pos_index_2];
      if (currentKernIndex == KernIndexs[i]) {
		  if (compare_str[posIndex]==' ') { *pos_kerns++ = tolower(Markers[i]); }
          else *pos_kerns++ = Markers[i];
      }
      else {
           i++;
           if (i>MAX_KERNEL_IN_ALI) {
             printf("ERROR !!! Too long alignment %d\n ", i );
             exit(1);
           }
         KernIndexs[i] = currentKernIndex;
         if (compare_str[posIndex]==' ') { *pos_kerns++ = tolower(Markers[i]);}
          else *pos_kerns++ = Markers[i];
      }
    }
    else {
     *pos_kerns++ = GAP_SYMBOL;
    }
   }
   else {
    *pos_kerns++ = GAP_SYMBOL;
   }
  }
  NumberKerns = i;

  fprintf(_f_out, "%s\n\n", str_kerns);
*/
/*
  // Print Kernels
  for (i = 1; i <= NumberKerns; i++) {
    wz = KernIndexs[i] - 1;
   	if(wz < 0) continue;
    //fprintf(_f_out, "%3d (%c) (%d,%d) len = %d score= %f\t Used_score= ", i, Markers[i], _kernels[wz].x, _kernels[wz].y, _kernels[wz].length, _kernels[wz].score);
	used_score = 0;

	//   _kernels[wz - 1]
	for (j = 0; j < _kernels[wz].length; j++) {
		c1 = amino_acids[_seq1->aas[_kernels[wz].x + j]];
		c2 = amino_acids[_seq2->aas[_kernels[wz].y + j]];
		
		if (_path_skop[(_kernels[wz].x + j)] == (_kernels[wz].y + j)){
				szUsed[j] = '-';
				used_score += subst_matrix[char_to_index[c1]][char_to_index[c2]];
		} else szUsed[j] = ' ';
		
		if (subst_matrix[char_to_index[c1]][char_to_index[c2]]<=0){
		//if (c1 != c2){
			c1 = tolower(c1);
			c2 = tolower(c2);
		}
		szDpi1[j] = c1;
		szDpi2[j] = c2;
		szUsedGS[j] = _path_GS[(_kernels[wz].x + j)] == (_kernels[wz].y + j) ? '_' : ' ';
	}
	szDpi1[j] = '\0';
	szDpi2[j] = '\0';
	szUsed[j] = '\0'; szUsedGS[j] = '\0';

    fprintf(_f_out, "%3d (%c) (%d,%d) len = %d score= %.2f\t Used_score= %.2f\n", i, Markers[i], _kernels[wz].x, _kernels[wz].y, _kernels[wz].length, _kernels[wz].score, used_score);
	fprintf(_f_out, "%s\n%s\n%s\n%s\n\n", szUsedGS, szDpi1, szDpi2, szUsed);

  }
*/
  fprintf(_f_out, "==========================================\n");

  return 0;
}


int island_compare_with_kernels2(sequence* _seq1, sequence* _seq2, 
								int* _path, int* _pathGS, 
								unsigned* _mask,
								kernel* _kernels, unsigned _kernel_count,
								islands_stat* isl_stat){

	unsigned	isl_len, isl_sim ;
	int			pos = 0, isl_nom = -1;
	char		is_isl = 0, is_kern;
	int			kern_ind, new_kern_ind , max_kern_ind, kern_count; 
	unsigned	len1 = _seq1->length, len2 = _seq2->length;
	score_type	isl_score, max_score, curr_max_score, add_score;
	score_type  kern_sum_weight, kern_weight, kern_max_weight ;

	while (pos<len1){ // go along path

		if (!is_isl){

			// begin of island
			if (_pathGS[pos] != -1) { 
				is_isl =1;
				isl_nom ++;

				// initiation of island statistic
				isl_len = 0;
				isl_sim = 0;
				isl_score = curr_max_score = 0;
				max_score = -100;       // to find max negative weight

				kern_count = 0;
				kern_sum_weight =  0;
				kern_weight = kern_max_weight = -100; // to find max negative weight
				kern_ind = new_kern_ind = max_kern_ind = -1 ;
				is_kern	  = 0;

				// proceed 1-st match
				isl_len++;
				add_score = subst_matrix[_seq1->aas[pos]][_seq2->aas[_pathGS[pos]]];
				isl_score += add_score;

				if (_path[pos] == _pathGS[pos]) isl_sim++;

				curr_max_score += add_score;
				if (curr_max_score > max_score) max_score = curr_max_score;
				if (curr_max_score < 0) curr_max_score = 0;

				// kernels 
				if (_mask[pos * len2 + _pathGS[pos]]) {
					is_kern = 1;
					new_kern_ind = _mask[pos * len2 + _pathGS[pos]]-1;//is_in_kernel(pos,_pathGS[pos],_kernels,_kernel_count);					  
					kern_sum_weight += add_score;
					
					if (new_kern_ind == kern_ind){ // the same kernel
						kern_weight += add_score;

					} else { // new kernel
						kern_count++;
						if (kern_weight > kern_max_weight){
							kern_max_weight = kern_weight;
							max_kern_ind = kern_ind;
						}

						kern_ind = new_kern_ind;
						kern_weight = add_score;
					}
				}
				 
			}
		} else {
			if ( (_pathGS[pos] == -1) || // end of island
				 (pos>0 && _pathGS[pos-1]!=-1 && _pathGS[pos-1]+1 != _pathGS[pos]) ) {

				is_isl=0;

				// save statistik for ended island
				isl_stat[isl_nom].isl_index = isl_nom;  
				isl_stat[isl_nom].isl_len   = isl_len;
				isl_stat[isl_nom].isl_sim   = isl_len ? (double) isl_sim/isl_len : 0;
				isl_stat[isl_nom].have_kern = is_kern;
				isl_stat[isl_nom].isl_score = isl_score;
				isl_stat[isl_nom].max_score = max_score;


				if (kern_weight > kern_max_weight){ // to control last kernel in island
					kern_max_weight = kern_weight;
					max_kern_ind = kern_ind;
				}

				isl_stat[isl_nom].kernel_count	  = kern_count;
				isl_stat[isl_nom].kern_max_weight = kern_max_weight;
				isl_stat[isl_nom].kern_sum_weight = kern_sum_weight;				
				isl_stat[isl_nom].kern_full_weight= (max_kern_ind!=-1) ? _kernels[max_kern_ind].score : -1;

				if (_pathGS[pos]!=-1) pos--; // if no DEL => return to the begin of island			
			} else { // continue island

				isl_len++;
				add_score = subst_matrix[_seq1->aas[pos]][_seq2->aas[_pathGS[pos]]];
				isl_score += add_score;

				if (_path[pos] == _pathGS[pos]) isl_sim++;

				curr_max_score += add_score;
				if (curr_max_score > max_score) max_score = curr_max_score;
				if (curr_max_score < 0) curr_max_score =0;

				// kernels 
				if (_mask[pos * len2 + _pathGS[pos]]) {
					is_kern = 1;
					new_kern_ind = _mask[pos * len2 + _pathGS[pos]]-1;//is_in_kernel(pos,_pathGS[pos],_kernels,_kernel_count);					  
					kern_sum_weight += add_score;
					
					if (new_kern_ind == kern_ind){ // the same kernel
						kern_weight += add_score;

					} else { // new kernel
						kern_count++;
						if (kern_weight > kern_max_weight){
							kern_max_weight = kern_weight;
							max_kern_ind = kern_ind;
						}

						kern_ind = new_kern_ind;
						kern_weight = add_score;
					}
				}
			} // end of islad continue
		}
		pos++;
	} // sequence end

	// save statistik for last unended island
	if (is_isl) {
		isl_stat[isl_nom].isl_index= isl_nom;  
		isl_stat[isl_nom].isl_len = isl_len;
		isl_stat[isl_nom].isl_sim = isl_len ? (double) isl_sim/isl_len : 0;
		isl_stat[isl_nom].have_kern = is_kern;
		isl_stat[isl_nom].isl_score =isl_score;
		isl_stat[isl_nom].max_score =max_score;

		if (kern_weight > kern_max_weight){ // to control last kernel in island
			kern_max_weight = kern_weight;
			max_kern_ind = kern_ind;
		}

		isl_stat[isl_nom].kernel_count	  = kern_count;
		isl_stat[isl_nom].kern_max_weight = kern_max_weight;
		isl_stat[isl_nom].kern_sum_weight = kern_sum_weight;				
		isl_stat[isl_nom].kern_full_weight= (max_kern_ind!=-1) ? _kernels[max_kern_ind].score : -1;
	}
	return isl_nom+1;
}

//-------------------------------------------------------
int is_in_kernel(coord_type _x, coord_type _y, 
					  kernel* _kernels, unsigned _kernel_count
					  ){

	unsigned i;

	for (i=0; i<_kernel_count; i++){
		if ( (_kernels[i].x <= _x) && (_x <= _kernels[i].x + _kernels[i].length))
			if ((_kernels[i].y <= _y) && ( _y <=_kernels[i].y + _kernels[i].length) &&
				( _y-_x == _kernels[i].y - _kernels[i].x) )
			{
				return i;
			}
	}

	return -1;
}
//-------------------------------------------------------
int	get_island_borders(int * _path, unsigned len, islands_stat* isl_stat){

	unsigned	pos = 0;
	unsigned	count = 0;
	char		is_isl = 0;

	while (pos<len){

		if (!is_isl){			
			if (_path[pos] != -1) { // begin of island
				is_isl =1;
				count++;
			}
		} else {
			if ( (_path[pos] == -1) || (_path[pos-1]!=-1 && _path[pos-1]+1 != _path[pos]) ) {// end of island
				is_isl=0;
				if (_path[pos]!=-1) pos--;
			}
		}
		pos++;
	}
	return count;

}

//-------------------------------------------------------
unsigned get_isl_count(int * _path, unsigned len){

	unsigned	pos = 0;
	unsigned	count = 0;
	char		is_isl = 0;

	while (pos<len){

		if (!is_isl){			
			if (_path[pos] != -1) { // begin of island
				is_isl =1;
				count++;
			}
		} else {
			if ( (_path[pos] == -1) || (_path[pos-1]!=-1 && _path[pos-1]+1 != _path[pos]) ) {// end of island
				is_isl=0;
				if (_path[pos]!=-1) pos--;
			}
		}
		pos++;
	}
	return count;
}

unsigned get_isl_count2(char* align_str_1, char* align_str_2){

	unsigned pos, len = strlen(align_str_1);
	unsigned isl_count = 0;
	char	 is_isl = 0;

	for (pos=0; pos<len; pos++){
		if ( !is_isl) {
			if ( align_str_1[pos] != GAP_SYMBOL && // begin of island
				 align_str_2[pos] != GAP_SYMBOL ){

				is_isl = 1;
				isl_count++;
			}
		} else {
			if ( align_str_1[pos] == GAP_SYMBOL || // end of island
				 align_str_2[pos] == GAP_SYMBOL ){
				
				is_isl = 0;
			}
		}
	}

	return isl_count;
}

//-----------------------------------------------------------------
void print_islands_statistic (char* file_name,char* family_name,
							  islands_stat* isl_stat, unsigned isl_count, 
							  sequence* _seq1, sequence* _seq2, 
							  double id_, double _acc, double _conf ){
	unsigned	i;
	FILE*		f;

	f = fopen(file_name, "at");
	if (!f) return;

	for (i=0; i<isl_count; i++){

	fprintf(f,"%s\t%s\t%s\t%.4f\t%.4f\t%.4f\t%d\t%d\t", 
			family_name,_seq1->name, _seq2->name,
			id_,_acc, _conf,
			_seq1->length, _seq2->length); 

		fprintf(f,"%d\t%d\t", 
				isl_stat[i].isl_index, isl_stat[i].isl_len); 
		fprintf(f,"%.3f\t%.3f\t", 
				isl_stat[i].isl_score, isl_stat[i].max_score);

		fprintf(f,"%.3f\t%d\t%d\t", 
				isl_stat[i].isl_sim,  isl_stat[i].have_kern,
				isl_stat[i].kernel_count );

		fprintf(f,"%.3f\t%.3f\t%.3f\t\n", 
				isl_stat[i].kern_max_weight, isl_stat[i].kern_sum_weight, 
				isl_stat[i].kern_full_weight);

	}
	//fprintf(f,"\n");
	fclose(f);
}
//-----------------------------------------------------------------
void print_max_islands_statistic (char* file_name,char* family_name,
							  islands_stat* isl_stat, unsigned isl_count, 
							  sequence* _seq1, sequence* _seq2, 
							  double id_,double _acc, double _conf ){
	unsigned	i;
	int			i_max = -1;
	FILE*		f;
	score_type	max_score = -1000;

	f = fopen(file_name, "at");
	if (!f) return;

	for (i=0; i<isl_count; i++){
		if (max_score < isl_stat[i].max_score){
			max_score = isl_stat[i].max_score;
			i_max = i;
		}
	}

	
	fprintf(f,"%s\t%s\t%s\t%.4f\t%.4f\t%.4f\t%d\t%d\t", 
			family_name,_seq1->name, _seq2->name,
			id_,_acc, _conf,
			_seq1->length, _seq2->length); 

	fprintf(f,"%d\t%d\t", 
			isl_stat[i_max].isl_index, isl_stat[i_max].isl_len); 
	fprintf(f,"%.3f\t%.3f\t", 
			isl_stat[i_max].isl_score, isl_stat[i_max].max_score);

	fprintf(f,"%.3f\t%d\t%d\t", 
			isl_stat[i_max].isl_sim,  isl_stat[i_max].have_kern,
			isl_stat[i_max].kernel_count );

	fprintf(f,"%.3f\t%.3f\t%.3f\t\n", 
			isl_stat[i_max].kern_max_weight, isl_stat[i_max].kern_sum_weight, 
			isl_stat[i_max].kern_full_weight);

	
	//fprintf(f,"\n");
	fclose(f);
}

//--------------------------------------------------------------
int island_compare_and_print(char* file_name, char* family_name,
							 sequence* _seq1, sequence* _seq2, 
								int* _path, int* _pathGS, 								
								unsigned* _mask,
								float id_,double _acc, double _conf,
								kernel* _kernels, unsigned _kernel_count
								){

  islands_stat* isl_stat = NULL;
  unsigned		isl_num, isl_num2;
  //char          align_str_1[MAX_LEN], align_str_2[MAX_LEN];

	isl_num = get_isl_count(_pathGS, _seq1->length);

	isl_stat = (islands_stat*) malloc(isl_num * sizeof(islands_stat));
	if (!isl_stat) return 0;

	isl_num2 = island_compare_with_kernels2 (_seq1, _seq2, _path, _pathGS, _mask,  _kernels, _kernel_count, isl_stat);
	assert(isl_num2 == isl_num);

	//print_islands_statistic (file_name, family_name, isl_stat , isl_num,_seq1, _seq2, id_);
	print_islands_statistic (
		file_name, family_name, isl_stat, isl_num,_seq1, _seq2,
		id_, _acc, _conf);
	free(isl_stat);

	return 1;
}
//--------------------------------------------------------------
int island_max_stat_and_print(char* file_name, char* family_name,
							 sequence* _seq1, sequence* _seq2, 
								int* _path, int* _pathGS, 								
								unsigned* _mask,
								double id_,double _acc, double _conf,
								kernel* _kernels, unsigned _kernel_count
								){

  islands_stat* isl_stat = NULL;
  unsigned		isl_num, isl_num2;
  //char          align_str_1[MAX_LEN], align_str_2[MAX_LEN];

	isl_num = get_isl_count(_pathGS, _seq1->length);

	isl_stat = (islands_stat*) malloc(isl_num * sizeof(islands_stat));
	if (!isl_stat) return 0;

	isl_num2 = island_compare_with_kernels2 (_seq1, _seq2, _path, _pathGS, _mask,  _kernels, _kernel_count, isl_stat);
	assert(isl_num2 == isl_num);

	//print_islands_statistic (file_name, family_name, isl_stat , isl_num,_seq1, _seq2, id_);
	print_max_islands_statistic (
		file_name, family_name, isl_stat, isl_num,
		_seq1, _seq2, id_, _acc, _conf );
	free(isl_stat);

	return 1;
}

int debug_compare_island_statistics (islands_stat* isl_stat1, islands_stat* isl_stat2,
									 unsigned	isl_count){

 unsigned i;

 for (i=0; i<isl_count; i++){
		assert(isl_stat1[i].isl_index == isl_stat2[i].isl_index );  
 }


 return 1;
}

unsigned ker_count_used_in_aln(kernel* kernels, unsigned kernel_count,
						       score_type weight, // we are interested only in kernels.score >= weight  
							   //unsigned* _ker_count, // number of such kernels
							   int* path,
							   unsigned path_len,
							   unsigned* mask, unsigned len2){

  unsigned used_ker = 0, i ;
  unsigned cur_ker = 0;

  for (i=0; i<path_len; i++){

	  if (path[i]!=-1 && mask[i * len2 + path[i]] && 
		  mask[i * len2 + path[i]] != cur_ker) {
		  
		  cur_ker = mask[i * len2 + path[i]];
		  if (kernels[cur_ker].score >= weight) used_ker++;
	  }
  }

  return used_ker;

}

//--------------------------------------------------
void print_seq_as_fasta(char* file_name, sequence* seq, char* open_file_mode){

	FILE*		f;
	unsigned	len = seq->length, k;
	char		 seq_print[51], *str_seq;

	f=fopen(file_name, open_file_mode);
	fprintf( f, ">%s length = %d\n",seq->name,len);

	str_seq= (char*)malloc((len+1)*sizeof(char)); 
	for (k=0; k<len; k++){
		str_seq[k] = amino_acids[seq->aas[k]];
	}
	str_seq[len] = '\0';

	for (k=0; k<len/50+1; k++) {
	  strncpy( seq_print, str_seq + k*50, 50);
	  seq_print[50] = '\0';
	  fprintf( f,"%s\n",seq_print);
	}

	fprintf( f,"\n");

	free(str_seq);
	fclose(f);
}
//---------------------------------------------------
int compare_with_pdb(sequence *seq, sw_params* _sw_params, 
					 int* pathres, sequence *seqPDB, FILE* f_out){ 

	//char        pdb_name[65] ;
	//sequence	seq_try, seqPDB, seq2;
  	//int			path1[MAX_LEN], path2[MAX_LEN];
	//char		strand;
	int			res = 0;
	char        align_str_1[MAX_LEN], align_str_2[MAX_LEN];

	
/*	strcpy(pdb_name,"pdb");
	 strncpy(pdb_name + 3, seq->name,4);
	 pdb_name[3+4]= '\0';
	 strcpy(pdb_name +7, ".ent"); 
	 if (strlen(seq->name)>4) strand = seq->name[4];
		else strand = ' ';

	if ((res = read_seq_PDB(pdb_name, seqPDB, strand)) != SUCCESS) {
		printf("error en readin PDB file %s\n", pdb_name);
		return res;
	}

	print_seq_as_fasta("test_fasta.txt", seqPDB);
*/
	//compute_sw_alignment_dblgap(
	compute_sw_alignment(
	  seqPDB,
	  seq,
	  _sw_params
	);
	
	format_alignment_output(
	  seqPDB,
	  seq,
	  G_sw_compute_data.path,
	  align_str_1,
	  align_str_2
	);

	fprintf(f_out, "%s <-> %s (SW) \n", seqPDB->name);
	fprintf(f_out, "%s\n%s\n\n", align_str_1, align_str_2);

	memcpy(pathres, G_sw_compute_data.path, sizeof(G_sw_compute_data.path));

	return SUCCESS;
}

//------------------------------------------------------
// create path via shot seq
void compare_with_pdb_revers(sequence *seq, sw_params* _sw_params, 
					  int* pathres2, sequence *seqPDB, FILE* f_out ){

	char        align_str_1[MAX_LEN], align_str_2[MAX_LEN];
/*	char        pdb_name[65] ;
	char		strand;
	int			res;

	
/*	strcpy(pdb_name,"pdb");
	 strncpy(pdb_name + 3, seq->name,4);
	 pdb_name[3+4]= '\0';
	 strcpy(pdb_name +7, ".ent"); 
	 if (strlen(seq->name)>4) strand = seq->name[4];
		else strand = ' ';

	if ((res = read_seq_PDB(pdb_name, seqPDB, strand)) != SUCCESS) {
		printf("error en readin PDB file %s\n", pdb_name);
	}

	print_seq_as_fasta("test_fasta.txt", seqPDB);
*/
	//compute_sw_alignment_dblgap(
	compute_sw_alignment(
	  seq,
		seqPDB,	  
	  _sw_params
	);
	
	format_alignment_output(
	  seq,
		seqPDB, 
	  G_sw_compute_data.path,
	  align_str_1,
	  align_str_2
	);

	fprintf(f_out, "%s <-> %s (SW) \n", seqPDB->name);
	fprintf(f_out, "%s\n%s\n\n", align_str_1, align_str_2);

	memcpy(pathres2, G_sw_compute_data.path, sizeof(G_sw_compute_data.path));
}

//
int fulfill_full_seqbank( malign_bank* short_bank, pairfull_bank* full_bank,
						 sw_params* _params,
						 char* pdb_file_path){

  int			path1[MAX_LEN], path2rev[MAX_LEN],path_sh[MAX_LEN], fpath[MAX_LEN] ;
  int			res ;
  unsigned		i/*, i_full*/, j, k,bank_size = short_bank->size, fullbank_size = bank_size;
  unsigned		len_sh, len_s;
  	char        pdb_name[65] ;
	char		strand;
	FILE*		f;

	f= fopen("no_pdb_file", "at");


	full_bank->name = (char*) malloc((strlen(short_bank->name)+1)*sizeof(char));
	strcpy(full_bank->name,short_bank->name);
	//full_bank->size = bank_size;
	full_bank->seqs = (sequence*)malloc(bank_size*sizeof(sequence));
	full_bank->paths_GS = (aln_path**)malloc(bank_size*sizeof(aln_path*));
	for (i = 0; i < bank_size; i++){
		full_bank->paths_GS[i] = (aln_path*)malloc(bank_size*sizeof(aln_path));
	}

	for (i=0; i< bank_size; i++){
		 strcpy(pdb_name, pdb_file_path);
		 strcat(pdb_name,"pdb");
		 len_s = strlen(pdb_name);
		 strncpy(pdb_name + len_s, short_bank->seqs[i].name,4);
		 pdb_name[len_s+4]= '\0';
		 strcpy(pdb_name +len_s+4, ".ent"); 
		 if (strlen(short_bank->seqs[i].name)>4) strand = short_bank->seqs[i].name[4];
			else strand = ' ';

		if ((res = read_seq_PDB(pdb_name, &(full_bank->seqs[i]), strand)) != SUCCESS) {
			if (res == -100) {
				//i_full--;  // not add the seq if no pdb_file for it  
				//fullbank_size --;
				fprintf(f," %s\n", pdb_name);
				delete_seq_from_malign_bank(short_bank, i);
				bank_size--;
				i--;

			} else {
				printf("error en readin PDB file %s\n", pdb_name);
				return res;
			}
		}
	}

	full_bank->size = bank_size;

		print_seq_as_fasta("seqs.txt", &full_bank->seqs[0], "wt");
		print_seq_as_fasta("seqs.txt", &short_bank->seqs[0], "at");
	for (i=1; i< bank_size; i++){
		print_seq_as_fasta("seqs.txt", &full_bank->seqs[i], "at");
		print_seq_as_fasta("seqs.txt", &short_bank->seqs[i], "at");
	}


    for(i = 0; i < bank_size-1; i++) {
		//compare_with_pdb(&short_bank->seqs[i], _sw_params,path1, &full_bank->seqs[i]);
		compute_sw_alignment(&full_bank->seqs[i],&short_bank->seqs[i], _params);		
		memcpy(path1, G_sw_compute_data.path, sizeof(G_sw_compute_data.path));

		save_aln_asFASTA(path1, &full_bank->seqs[i],&short_bank->seqs[i], '.');

		for(j = i + 1; j < bank_size; j++) {
			//compare_with_pdb_revers(&short_bank->seqs[j], _sw_params,path2rev,&full_bank->seqs[j]);
			compute_sw_alignment(&short_bank->seqs[j], &full_bank->seqs[j], _params);
			memcpy(path2rev, G_sw_compute_data.path, sizeof(G_sw_compute_data.path));
	
			save_aln_asFASTA(path2rev, &short_bank->seqs[j], &full_bank->seqs[j], '.');
			
			format_alignment_path(
			  short_bank->aligned_seq_strings[i],
			  short_bank->aligned_seq_strings[j],
			  path_sh,
			  &len_sh
			);

			save_aln_asFASTA(path_sh,&short_bank->seqs[i], &short_bank->seqs[j], '.');

			for(k=0; k < full_bank->seqs[i].length; k++){
				if (path1[k]==-1) fpath[k] = -1;
				else {
					assert(path1[k] < len_sh);
					if (path_sh[path1[k]] == -1)fpath[k] = -1; 
					else {					
						assert(path_sh[path1[k]] < short_bank->seqs[j].length);
						fpath[k]= path2rev[path_sh[path1[k]]];
					}
				}
			}
			//debug_print_path(path1, full_bank->seqs[i].length, "paths.txt");
			//debug_print_path(path_sh, len_sh, "paths.txt");
			//debug_print_path(path2rev, full_bank->seqs[j].length, "paths.txt");
			//debug_print_path(fpath, full_bank->seqs[i].length, "paths.txt");

			full_bank->paths_GS[i][j].length = full_bank->seqs[i].length;

			memcpy(full_bank->paths_GS[i][j].path, fpath,full_bank->seqs[i].length * sizeof(int));
		 }
	}

	print_full_seqbank(full_bank, "res_bank");
	malign_bank_release(short_bank);
	fclose(f);
	return SUCCESS;
}

// free all the memory that was allocated by full_seqbank
//--------------------------------------------
void free_full_seqbank(pairfull_bank* _bank){

	free(_bank->name);
	while(_bank->size--) {
		free(_bank->seqs[_bank->size].aas);
		free(_bank->seqs[_bank->size].name);
		free(_bank->paths_GS[_bank->size]);
	}
	free(_bank->seqs);
	free(_bank->paths_GS);
}


void print_full_seqbank(pairfull_bank* full_bank, char* file_name){

	unsigned	i,j;
	FILE*		f;
    char        align_str_1[MAX_LEN],
                align_str_2[MAX_LEN];

	f= fopen(file_name, "wt");

	if(!f) return;
	fprintf(f,"bank_name = %s\n\n", full_bank->name);

	for(i=0; i<full_bank->size-1; i++){
	
		for(j=i+1; j<full_bank->size; j++){

			format_alignment_output(
			  &full_bank->seqs[i],
			  &full_bank->seqs[j],
			  full_bank->paths_GS[i][j].path,
			  align_str_1,
			  align_str_2
			);

			fprintf(f,"%-12s %s\n%-12s %s\n\n",full_bank->seqs[i].name, align_str_1,
											   full_bank->seqs[j].name, align_str_2);

		}
	}
	fclose(f);
}

void debug_print_path(int* path, unsigned len, char* file_name){
	unsigned	i;
	FILE*		f;

	f= fopen(file_name, "at");
	if(!f) return;

	for (i=0; i<len; i++) fprintf(f,"%d\t", i);
	fprintf(f,"\n");
	for (i=0; i<len; i++) fprintf(f,"%d\t", path[i]);
	fprintf(f,"\n\n");
	fclose(f);
}

//---------------------------------------------------------------------------
// procedures for structure comparing
//---------------------------------------------------------------------------

// save alignment in FASTA format in name1_name2_aln.txt file 
int save_aln_asFASTA(int* path, sequence *seq1, sequence *seq2, char del_symbol){

	unsigned i, j, m = seq1->length, i_pos = 0, j_pos = 0, pos=0;
	char     align1[MAX_LEN], align2[MAX_LEN], file_name[256];
	FILE*	 f;

	assert(seq1);
	assert(seq2);
	assert(align1);
	assert(align2);

	for(i = 0; i < m; i++) {
		j = path[i];
		// reach the (i, j)
		if(j == -1) {
			align1[pos] = amino_acids[seq1->aas[i]];
			align2[pos] = del_symbol;
			pos++;
		} else {
			while(j_pos < j)  {   // reach the j
				align1[pos] = del_symbol;
				align2[pos] = amino_acids[seq2->aas[j_pos++]];
				pos++;
			}
			align1[pos] = amino_acids[seq1->aas[i]];
			align2[pos] = amino_acids[seq2->aas[j_pos++]];
			pos++;
		}
	}
	while(j_pos < seq2->length) {
		align1[pos] = del_symbol;
		align2[pos] = amino_acids[seq2->aas[j_pos++]];
		pos++;
	}
	align1[pos] = '\0';
	align2[pos] = '\0';


	strcpy(file_name, seq1->name);
	strcat(file_name, "_");
	strcat(file_name, seq2->name);
	strcat(file_name, "_aln.txt");

	f = fopen(file_name,"wt");
	if (!f) return E_FILEERROR;

	fprintf(f,"> %s \n%s\n",seq1->name, align1);
	fprintf(f,"> %s \n%s\n",seq2->name, align2);

	fclose(f);

	return SUCCESS;
}

double calculate_distance (sequence *seq1, sequence *seq2, int* path, double* dists,
						   char* pdb_file_path){

	char	command_line[1024], str[516];
	char	file1[32], file2[32], aln_f_name[256];
	char	strand1, strand2;
	unsigned len, pos=0, i;
	char*	sh;
	FILE*	f;
	double	temp_dist[MAX_LEN];
	double	sr_dist = 0.0f;

	if (save_aln_asFASTA(path, seq1,seq2,'-') != SUCCESS) {
		return E_FILEERROR;
	}

	make_pdb_filename(seq1->name, file1, &strand1);
	make_pdb_filename(seq2->name, file2, &strand2);

	strcpy(aln_f_name, seq1->name);
	strcat(aln_f_name, "_");
	strcat(aln_f_name, seq2->name);
	strcat(aln_f_name, "_aln.txt");

	strcpy(command_line, "Alignment.exe \"");
	strcat(command_line, pdb_file_path);
	strcat(command_line, file1);
	strcat(command_line, "\" \"");
	
	strcat(command_line, pdb_file_path);
	strcat(command_line, file2);
	strcat(command_line, "\" \"");
	strcat(command_line, aln_f_name);
	strcat(command_line, "\" \"");
	len = strlen(command_line);
	//command_line[len++] = '"';
	command_line[len++] = strand1;
	command_line[len++] = '"';
	command_line[len++] = ' ';
	command_line[len++] = '"';
	command_line[len++] = strand2;
	command_line[len++] = '"';
	command_line[len++] = '\0';

	strcat(command_line, " >  res.pdb\n");
	// start of Alignment.exe programm
	system(command_line);
	// end

	/*strncpy(aln_f_name,file1+3,7);
	aln_f_name[7]  = '_' ;
	aln_f_name[8]  = '\0';
	strncat(aln_f_name,file2+3,7);
	aln_f_name[15] = '\0';*/
	strcpy(aln_f_name, file1+3);
	strcat(aln_f_name, "_");
	strcat(aln_f_name, file2+3);

	strcat(aln_f_name, ".log");

	f=fopen(aln_f_name,"rt");
	if (!f) return E_FILEERROR;

	fgets(str, 255, f);
	while (!strstr(str, "Squared distance")) fgets(str, 255, f);
	fgets(str, 255, f);fgets(str, 255, f);

	memset(dists, -1, sizeof(dists));
	i=0;
	while ( strstr(str,"Record") && !strstr(str,"Weights")){
		sh = strtok(str,"\t\n");
		sh = strtok(NULL,"\t\n");
		temp_dist[pos] = atof(sh);
		while (path[i] == -1) i++;
		
		assert(i < seq1->length);

		dists[i] = atof(sh); 
		pos++;
		fgets(str, 255, f);
		i++;
	}
	fclose(f);

	for (i=0; i<pos; i++) sr_dist += temp_dist[i]; 
	return pos ? sr_dist/pos: -1;
}

void make_pdb_filename(char* seq_name, char* res_file_name, char* strand){

	int i;

	strcpy(res_file_name,"pdb");
	for (i=0; i<4; i++){
		seq_name[i] = tolower(seq_name[i]);
	}
	strncpy(res_file_name + 3, seq_name,4);
	res_file_name[3+4]= '\0';
	strcat(res_file_name, ".ent"); 
	if (strlen(seq_name)>4) *strand = toupper(seq_name[4]);
		else *strand = ' ';

}

//---------------------------------------------------------------------
// Calculates alignment identity _path_GS path of Golden standard alignment
// _path_sample - path of program alignment of he same sequences
double calc_dist_alnide(int* path_GS, double* dist_GS, 
						int* path,    double* dist, 
						unsigned _len_GS)
{
	unsigned i, count = 0, match = 0;
	double	 all_dist = 0, match_dist = 0;

	for(i = 0; i < _len_GS; i++) {
		if(path_GS[i] != -1) {
			if(path[i] == path_GS[i]) {
				match ++ ;
				match_dist += ((dist_GS[i]+dist[i]) > 0.02) ? 2/(dist_GS[i]+dist[i]) : 100;
				all_dist += ((dist_GS[i]+dist[i]) > 0.02) ? 2/(dist_GS[i]+dist[i]) : 100;
			} else all_dist += (dist_GS[i] > 0.01) ? 1/(dist_GS[i]) : 100;
			count++;
		}	
	}
	return count ? (double)match_dist / all_dist : 0.00f;
}

//---------------------------------------------------------------------
// Calculates alignment identity in kernels. _path_GS  - path of Golden standard alignment
// _path_sample - path of program alignment of he same sequences
double calc_dist_kernel_alnide(int* path_GS, double* dist_GS, 
							   int* path,    double* dist,
							   unsigned _len_GS, unsigned* _mask, unsigned _length2)
{
	unsigned i, count = 0, match = 0;
	double	 all_dist = 0, match_dist = 0;

	for(i = 0; i < _len_GS; i++) {
		if(path_GS[i] != -1 && _mask[i * _length2 + path_GS[i]]) {
			if(path[i] == path_GS[i]) {
				match ++ ;
				match_dist += ((dist_GS[i]+dist[i]) > 0.02) ? 2/(dist_GS[i]+dist[i]) : 100;
				all_dist += ((dist_GS[i]+dist[i]) > 0.02) ? 2/(dist_GS[i]+dist[i]) : 100;

			} else all_dist += (dist_GS[i] > 0.01) ? 1/(dist_GS[i]) : 100;
			count++;
			
		}	
	}
	return count ? (double)match_dist / all_dist : 0.00f;
}


//---------------------------------------------------------------------
// Calculates alignment identity _path_GS path of Golden standard alignment
// _path_sample - path of program alignment of the same sequences
double calc_dist_footstep_alnide(
						int* path_GS, double* dist_GS, 
						int* path,    double* dist, 
						unsigned _len_GS,
						double Dist)
{
	unsigned i, count = 0, match = 0;
	double	 all_dist = 0, match_dist = 0, d;

	for(i = 0; i < _len_GS; i++) {
		if(path_GS[i] != -1) {
			if(path[i] == path_GS[i]) {
				d = 0.5f * (dist_GS[i]+dist[i]);
				match ++ ;
				match_dist += (d<Dist)? 0.5f*(1+sqrt(1-d/Dist)) : 0.5f*Dist/d;
				all_dist += (d<Dist)? 0.5f*(1+sqrt(1-d/Dist)) : 0.5f*Dist/d;
			} else
				all_dist += (dist_GS[i]<Dist)? 0.5f*(1+sqrt(1-dist_GS[i]/Dist)) : 0.5f*Dist/dist_GS[i];
			count++;
		}	
	}
	return count ? (double)match_dist / all_dist : 0.00f;
}

//---------------------------------------------------------------------
// Calculates alignment identity in kernels. _path_GS  - path of Golden standard alignment
// _path_sample - path of program alignment of he same sequences
double calc_dist_footstep_kernel_alnide(
							   int* path_GS, double* dist_GS, 
							   int* path,    double* dist,
							   unsigned _len_GS, unsigned* _mask, unsigned _length2,
							   double Dist)
{
	unsigned i, count = 0, match = 0;
	double	 all_dist = 0, match_dist = 0, d;

	for(i = 0; i < _len_GS; i++) {
		if(path_GS[i] != -1 && _mask[i * _length2 + path_GS[i]]) {
			if(path[i] == path_GS[i]) {
				d = 0.5f * (dist_GS[i]+dist[i]); 
				match ++ ;
				match_dist += (d<Dist)? 0.5f*(1+sqrt(1-d/Dist)) : 0.5f*Dist/d;
				all_dist += (d<Dist)? 0.5f*(1+sqrt(1-d/Dist)) : 0.5f*Dist/d;
			} else all_dist += (dist_GS[i]<Dist)? 0.5f*(1+sqrt(1-dist_GS[i]/Dist)) : 0.5f*Dist/dist_GS[i];
			count++;
		}	
	}
	return count ? (double)match_dist / all_dist : 0.00f;
}

//---------------------------------------------------------------------
// Calculates alignment identity _path_GS path of Golden standard alignment
// _path_sample - path of program alignment of he same sequences
double calc_dist_footstep_conf(
						int* path_GS, double* dist_GS, 
						int* path,    double* dist, 
						unsigned _len,
						double Dist)
{
	unsigned i, count = 0, match = 0;
	double	 all_dist = 0, match_dist = 0, d;

	for(i = 0; i < _len; i++) {
		if(path[i] != -1) {
			if(path[i] == path_GS[i]) {
				d = 0.5f * (dist_GS[i]+dist[i]);
				match ++ ;
				match_dist += (d<Dist)? 0.5f*(1+sqrt(1-d/Dist)) : 0.5f*Dist/d;
				all_dist += (d<Dist)? 0.5f*(1+sqrt(1-d/Dist)) : 0.5f*Dist/d;
			} else all_dist +=1 ;
			count++;
		}	
	}
	return count ? (double)match_dist / all_dist : 0.00f;
}

//---------------------------------------------------------------------
// Calculates alignment identity in kernels. _path_GS  - path of Golden standard alignment
// _path_sample - path of program alignment of he same sequences
double calc_dist_footstep_kernel_conf(
							   int* path_GS, double* dist_GS, 
							   int* path,    double* dist,
							   unsigned _len, unsigned* _mask, unsigned _length2,
							   double Dist)
{
	unsigned i, count = 0, match = 0;
	double	 all_dist = 0, match_dist = 0, d;

	for(i = 0; i < _len; i++) {
		if(path[i] != -1 && _mask[i * _length2 + path[i]]) {
			if(path[i] == path_GS[i]) {
				d = 0.5f * (dist_GS[i]+dist[i]); 
				match ++ ;
				match_dist += (d<Dist)? 0.5f*(1+sqrt(1-d/Dist)) : 0.5f*Dist/d;
				all_dist += (d<Dist)? 0.5f*(1+sqrt(1-d/Dist)) : 0.5f*Dist/d;
			} else all_dist += 1;
			count++;
		}	
	}
	return count ? (double)match_dist / all_dist : 0.00f;
}

// 
void get_scop_code( char* seq_name, char* code){

	char buff[1000];
	char* delims = " \t";
	char *res;
    char *pdest;

	strcpy(buff, seq_name);

	res = strtok(buff, delims);
	res = strtok(NULL,delims);
	strcpy(code, res);

	pdest = strrchr( res, '.' );
	res[pdest - res ] = '\0';
	strcpy(code, res);
}
// avalable only for scop.res database
// return number of proteins with same superfamily as query  
int get_bank_Nrel(fasta_bank * _bank, char* seq_name){

	int     i,resN = 0;
	char    code[10];
	char*	delims = " \t";

	get_scop_code(seq_name, code);

	for ( i=0; i<_bank->size; i++){
		if (strstr(_bank->seqs[i].name, code)) resN++;
	}
	return resN;
}

//  read result list of FASTA serch
int read_FASTA_search_file(char*     res_file_name,
						   unsigned  Nrel,
						   char*	 query_name,
						   float*    sort_skop,
						   float*	 n1,
						   float*	 n2,
						   unsigned* tot){

	FILE*    f;
	char	 buff[512];
	char     code[10];
	char	 list[3003];
	unsigned numb = 0;

	f = fopen(res_file_name, "rt");
	if(!f) {
		printf("Can't open FASTA result file %s \n",res_file_name); 
		return E_FILEERROR;
	}

	fgets(buff,511,f); 
	while (!feof(f) && !strstr(buff, "The best scores are"))
		fgets(buff, 511,f);
	if (feof(f)) {
		printf("Error in reading FASTA result file %s \n",res_file_name);
		return E_FILEERROR;
	}

	get_scop_code(query_name, code);
	memset(list, 0, 3001);

	fgets(buff,511, f); // skip first rezult
	fgets(buff,511, f);
	while (!feof(f) && buff[0] != '\n'){
		if (strstr(buff,code)) list[numb] = 1;
			else list[numb] = 0;
		numb++;
		if (numb>3000) {
			printf("Too long list of best hits in FASTA result file %s \n",res_file_name);
			break;
		}
		fgets(buff,511, f);
	}
	fclose(f);
	(*sort_skop) =estimate_sort_skop(Nrel,numb,list);
	(*n1) = estimate_n1(Nrel,numb,list);
	(*n2) =	estimate_n2(Nrel,numb,list);
	(*tot) = numb;	
	
	return SUCCESS;
}

//  read result list of BLAST serch
int read_BLAST_search_file(char*     res_file_name,
						   unsigned  Nrel,
						   char*	 query_name,
						   float*    sort,
						   float*	 n1,
						   float*	 n2,
						   unsigned* tot){
	FILE*    f;
	char	 buff[512];
	char     code[10];
	char	 list[2000];
	unsigned numb = 0;

	f = fopen(res_file_name, "rt");
	if(!f) {
		printf("Can't open BLAST result file %s \n",res_file_name); 
		return E_FILEERROR;
	}

	fgets(buff,511,f); 
	while (!feof(f) && !strstr(buff, "Sequences producing significant alignments"))
		fgets(buff, 511,f);
	if (feof(f)) {
		printf("Error in reading BLAST result file %s \n",res_file_name);
		return E_FILEERROR;
	}

	get_scop_code(query_name, code);
	memset(list, 0, 2000);

	fgets(buff,511, f);fgets(buff,511, f);
	fgets(buff,511, f); // skip first rezult
	while (!feof(f) && buff[0] != '\n'){
		if (strstr(buff,code)) list[numb] = 1;
			else list[numb] = 0;
		numb++;
		if (numb>2000) {
			printf("Too long list of best hits in FASTA result file %s \n",res_file_name);
			break;
		}
		fgets(buff,511, f);
	}
	fclose(f);
	(*sort) =estimate_sort_skop(Nrel,numb,list);
	(*n1) = estimate_n1(Nrel,numb,list);
	(*n2) =	estimate_n2(Nrel,numb,list);
	(*tot) = numb;	

	return SUCCESS;
}

// estimate number of true results in  first results
float estimate_n1(unsigned  Nrel,
				  unsigned  numb,
				  char*	    list){

	unsigned i, n1=0;
	unsigned num = (numb > Nrel) ? Nrel : numb;
	
	if (numb==0) return 0;
		
	for ( i=0; i<num; i++){
		if (list[i] == 1) n1++;
	}
	return (Nrel>0) ? (float)n1/Nrel : 0;
}

// estimate position of first error in list
float estimate_n2(unsigned  Nrel,
				  unsigned  numb,
				  char*	    list){
	unsigned i, n2=0;

	if (numb==0) return 0;

	for (i=0; i<numb; i++){

		if (list[i] == 0) {
			n2 = i+1;
			break;
		}
	}
	if (n2==0) n2 = numb-1;
		else n2 = n2-1;
	return (Nrel>0) ? (float)n2/Nrel : 0;
}

// estimate complicate formula of accuracy of serch list
float estimate_sort_skop(unsigned  Nrel,
						 unsigned  numb,
						 char*	   list){

		unsigned i, sort = 0;
		float    best = (float)(numb-0.5*(Nrel-1))*Nrel;
		unsigned numbi;

	    if (numb==0) return 0;

		if (Nrel>numb) {
			best =(float)(Nrel-0.5*(Nrel-1))*Nrel; 
			numbi = Nrel;
		} else numbi = numb;

		for (i=0; i< numb; i++){
			sort += list[i]*(numbi-i);
		}

		return (best > 0) ? (float)sort/best : 0; 
}

// print used parametrs in res file
void print_sw_params_in_rezult( FILE* res_file,	sw_params*	_params, char* matrix_name){

	fprintf(res_file, "used procedure of SW alignment\n matrix = %s\n",matrix_name); 
	fprintf(res_file,"sw_GEP = %d\nsw_GOP = %d\n\n",_params->GEP, _params->GOP);
}

void print_alignment(char* name1,char* name2,
					  char* align_str_1,char* align_str_2,
					  float ID, score_type weight,
					  FILE * f){

	char n1[6], n2[6];
	strncpy(n1, name1, 5); n1[6] = '\0';	
	strncpy(n2, name2, 5); n2[6] = '\0';

	fprintf(f, "%5s - %5s, ID = %.4f, W = %d \n",
		n1, n2, ID, weight);
	fprintf(f, "%5s %s\n%5s %s\n\n",
		n1, align_str_1,n2, align_str_2); 

}

void convert_to_fasta( sequence * seq){

	int i;
	for ( i=0; i<seq->length ; i++){
		seq->aas[i]++;
	}
	seq->aas = realloc(seq->aas, (seq->length + 1)* sizeof (char));
	seq->aas[seq->length] = '\0'; 
}

void convert_to_fasta2( sequence * seq){

	int i;
	for ( i=0; i<seq->length ; i++){
		seq->aas[i]++;
	}
	//seq->aas = realloc(seq->aas, (seq->length + 1)* sizeof (char));
	seq->aas[seq->length] = '\0'; 
}


void copy_seq(sequence * init_seq, sequence * res_seq){

	res_seq->length = init_seq->length;
	memcpy(res_seq->aas, init_seq->aas, (res_seq->length) * sizeof(char));
	strcpy(res_seq->name, init_seq->name);
}

void get_mem_for_seq(sequence * seq){
  seq->aas = (char*) malloc(sizeof(char) * MAX_LEN);
  seq->name =(char*) malloc(sizeof(char) * 60);
}

///-------------------------------------------------------------------------
void init_seq_matrix( sequence * seq1){ // change seq_matrix 
										// need fulfilled subst_matrix
	char		*aas = seq1->aas;
	unsigned	i,j, len = seq1->length;

	for (j=0; j<ALPHABET_SIZE; j++)
	  for (i=0; i<len; i++){
		seq_matrix[i + j*len] = subst_matrix[j][aas[i]];
		//seq_matrix++;
	  }
}

//---------------------------------------------------------------------------

void convert_seq2path_to_seq1path(sequence* seq1,sequence* seq2,
								  int* path1, int* path2){


	unsigned i=0,j=0, len1 = seq1->length, len2 = seq2->length;
	

	if (path2[0]>0) 
		for (;i<path2[0]; i++) path1[i] = -1;

	for (j=0; j<len2;j++){

		if (path2[j]==-1) continue;
		if (path2[j] == i){
			path1[i] = j;
			i++;
		} else {
			path1[i] = -1;
			i++;
		}

	}
}

//    
//     

score_type Calc_weight (char* aln_seq1,
						char* aln_seq2,
						char* Matrix_file,
						score_type GOP,
						score_type GEP,
						char* Init_Matrix
						){

  unsigned				i;
  score_type			weight;
  char					is_gap = 0, aln_beg =0;
  int					ind1, ind2;
  char					s1,s2;
  int					last_gap;



	if(matrix_read_from_file(Matrix_file) != SUCCESS) {
		printf("Error reading substitution matrix\n");
		return -1;
	}

  weight = 0.0f;
  for( i=0; i< strlen(aln_seq1); i++){

	  s1 = toupper(aln_seq1[i]);
	  s2 = toupper(aln_seq2[i]);

	  if ( s1!= GAP_SYMBOL && s2!= GAP_SYMBOL ) {
		  is_gap = 0; last_gap =0; aln_beg =1;
		  ind1 = char_to_index[s1];
		  ind2 = char_to_index[s2];
		  weight += subst_matrix[ind1][ind2];
	  }

	  if (aln_beg && is_gap && (s1 == GAP_SYMBOL || s2 == GAP_SYMBOL)) {
		  weight -= GEP;
		  last_gap++;
	  }

	  if (aln_beg && !is_gap && (s1 == GAP_SYMBOL || s2 == GAP_SYMBOL)) {
		  is_gap = 1;
		  weight -= GEP + GOP;
		  last_gap = 1;
	  }
  } 
  if (is_gap) {
	  weight += GEP*last_gap + GOP;
  }

	if(matrix_read_from_file(Init_Matrix) != SUCCESS) {
		printf("Error reading substitution matrix\n");
		return -1;
	}

	return weight;
}

//    
//     
//     ,    .
//  subst2_matrix

score_type Calc_weight_subst2 (char* aln_seq1,
						char* aln_seq2,
						score_type GOP,
						score_type GEP){

  unsigned				i;
  score_type			weight;
  char					is_gap = 0, aln_beg =0;
  int					ind1, ind2;
  char					s1,s2;
  int					last_gap;


  weight = 0.0f;
  for( i=0; i< strlen(aln_seq1); i++){

	  s1 = toupper(aln_seq1[i]);
	  s2 = toupper(aln_seq2[i]);

	  if ( s1!= GAP_SYMBOL && s2!= GAP_SYMBOL ) {
		  is_gap = 0; last_gap =0; aln_beg =1;
		  ind1 = char_to_index[s1];
		  ind2 = char_to_index[s2];
		  weight += subst2_matrix[ind1][ind2];
	  }

	  if (aln_beg && is_gap && (s1 == GAP_SYMBOL || s2 == GAP_SYMBOL)) {
		  weight -= GEP;
		  last_gap++;
	  }

	  if (aln_beg && !is_gap && (s1 == GAP_SYMBOL || s2 == GAP_SYMBOL)) {
		  is_gap = 1;
		  weight -= GEP + GOP;
		  last_gap = 1;
	  }
  } 
  if (is_gap) {
	  weight += GEP*last_gap + GOP;
  }

  return weight;
}